# -*- coding: utf-8 -*-
#
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import importlib
import sys
from itertools import chain

from google.api_core.protobuf_helpers import get_messages

from google.ads.google_ads import util


if sys.version_info < (3, 6):
    raise ImportError("This module requires Python 3.6 or later.")


_lazy_name_to_package_map = {
    'ad_asset_pb2':"google.ads.google_ads.v4.proto.common",
    'ad_type_infos_pb2':"google.ads.google_ads.v4.proto.common",
    'asset_types_pb2':"google.ads.google_ads.v4.proto.common",
    'bidding_pb2':"google.ads.google_ads.v4.proto.common",
    'click_location_pb2':"google.ads.google_ads.v4.proto.common",
    'criteria_pb2':"google.ads.google_ads.v4.proto.common",
    'criterion_category_availability_pb2':"google.ads.google_ads.v4.proto.common",
    'custom_parameter_pb2':"google.ads.google_ads.v4.proto.common",
    'dates_pb2':"google.ads.google_ads.v4.proto.common",
    'explorer_auto_optimizer_setting_pb2':"google.ads.google_ads.v4.proto.common",
    'extensions_pb2':"google.ads.google_ads.v4.proto.common",
    'feed_common_pb2':"google.ads.google_ads.v4.proto.common",
    'final_app_url_pb2':"google.ads.google_ads.v4.proto.common",
    'frequency_cap_pb2':"google.ads.google_ads.v4.proto.common",
    'keyword_plan_common_pb2':"google.ads.google_ads.v4.proto.common",
    'matching_function_pb2':"google.ads.google_ads.v4.proto.common",
    'metrics_pb2':"google.ads.google_ads.v4.proto.common",
    'offline_user_data_pb2':"google.ads.google_ads.v4.proto.common",
    'policy_pb2':"google.ads.google_ads.v4.proto.common",
    'real_time_bidding_setting_pb2':"google.ads.google_ads.v4.proto.common",
    'segments_pb2':"google.ads.google_ads.v4.proto.common",
    'simulation_pb2':"google.ads.google_ads.v4.proto.common",
    'tag_snippet_pb2':"google.ads.google_ads.v4.proto.common",
    'targeting_setting_pb2':"google.ads.google_ads.v4.proto.common",
    'text_label_pb2':"google.ads.google_ads.v4.proto.common",
    'url_collection_pb2':"google.ads.google_ads.v4.proto.common",
    'user_lists_pb2':"google.ads.google_ads.v4.proto.common",
    'value_pb2':"google.ads.google_ads.v4.proto.common",
    'access_reason_pb2':"google.ads.google_ads.v4.proto.enums",
    'access_role_pb2':"google.ads.google_ads.v4.proto.enums",
    'account_budget_proposal_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'account_budget_proposal_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'account_budget_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'account_link_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'ad_customizer_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'ad_group_ad_rotation_mode_pb2':"google.ads.google_ads.v4.proto.enums",
    'ad_group_ad_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'ad_group_criterion_approval_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'ad_group_criterion_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'ad_group_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'ad_group_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'ad_network_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'ad_serving_optimization_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'ad_strength_pb2':"google.ads.google_ads.v4.proto.enums",
    'ad_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'advertising_channel_sub_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'advertising_channel_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'affiliate_location_feed_relationship_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'affiliate_location_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'age_range_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'app_campaign_app_store_pb2':"google.ads.google_ads.v4.proto.enums",
    'app_campaign_bidding_strategy_goal_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'app_payment_model_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'app_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'app_store_pb2':"google.ads.google_ads.v4.proto.enums",
    'app_url_operating_system_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'asset_field_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'asset_performance_label_pb2':"google.ads.google_ads.v4.proto.enums",
    'asset_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'attribution_model_pb2':"google.ads.google_ads.v4.proto.enums",
    'batch_job_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'bid_modifier_source_pb2':"google.ads.google_ads.v4.proto.enums",
    'bidding_source_pb2':"google.ads.google_ads.v4.proto.enums",
    'bidding_strategy_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'bidding_strategy_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'billing_setup_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'brand_safety_suitability_pb2':"google.ads.google_ads.v4.proto.enums",
    'budget_delivery_method_pb2':"google.ads.google_ads.v4.proto.enums",
    'budget_period_pb2':"google.ads.google_ads.v4.proto.enums",
    'budget_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'budget_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'call_conversion_reporting_state_pb2':"google.ads.google_ads.v4.proto.enums",
    'call_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'callout_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'campaign_criterion_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'campaign_draft_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'campaign_experiment_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'campaign_experiment_traffic_split_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'campaign_experiment_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'campaign_serving_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'campaign_shared_set_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'campaign_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'change_status_operation_pb2':"google.ads.google_ads.v4.proto.enums",
    'change_status_resource_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'click_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'content_label_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'conversion_action_category_pb2':"google.ads.google_ads.v4.proto.enums",
    'conversion_action_counting_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'conversion_action_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'conversion_action_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'conversion_adjustment_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'conversion_attribution_event_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'conversion_lag_bucket_pb2':"google.ads.google_ads.v4.proto.enums",
    'conversion_or_adjustment_lag_bucket_pb2':"google.ads.google_ads.v4.proto.enums",
    'criterion_category_channel_availability_mode_pb2':"google.ads.google_ads.v4.proto.enums",
    'criterion_category_locale_availability_mode_pb2':"google.ads.google_ads.v4.proto.enums",
    'criterion_system_serving_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'criterion_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'custom_interest_member_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'custom_interest_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'custom_interest_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'custom_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'customer_match_upload_key_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'customer_pay_per_conversion_eligibility_failure_reason_pb2':"google.ads.google_ads.v4.proto.enums",
    'data_driven_model_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'day_of_week_pb2':"google.ads.google_ads.v4.proto.enums",
    'device_pb2':"google.ads.google_ads.v4.proto.enums",
    'display_ad_format_setting_pb2':"google.ads.google_ads.v4.proto.enums",
    'display_upload_product_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'distance_bucket_pb2':"google.ads.google_ads.v4.proto.enums",
    'dsa_page_feed_criterion_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'education_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'extension_setting_device_pb2':"google.ads.google_ads.v4.proto.enums",
    'extension_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'external_conversion_source_pb2':"google.ads.google_ads.v4.proto.enums",
    'feed_attribute_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'feed_item_quality_approval_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'feed_item_quality_disapproval_reason_pb2':"google.ads.google_ads.v4.proto.enums",
    'feed_item_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'feed_item_target_device_pb2':"google.ads.google_ads.v4.proto.enums",
    'feed_item_target_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'feed_item_target_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'feed_item_validation_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'feed_link_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'feed_mapping_criterion_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'feed_mapping_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'feed_origin_pb2':"google.ads.google_ads.v4.proto.enums",
    'feed_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'flight_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'frequency_cap_event_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'frequency_cap_level_pb2':"google.ads.google_ads.v4.proto.enums",
    'frequency_cap_time_unit_pb2':"google.ads.google_ads.v4.proto.enums",
    'gender_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'geo_target_constant_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'geo_targeting_restriction_pb2':"google.ads.google_ads.v4.proto.enums",
    'geo_targeting_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'google_ads_field_category_pb2':"google.ads.google_ads.v4.proto.enums",
    'google_ads_field_data_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'hotel_date_selection_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'hotel_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'hotel_price_bucket_pb2':"google.ads.google_ads.v4.proto.enums",
    'hotel_rate_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'income_range_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'interaction_event_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'interaction_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'invoice_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'job_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'keyword_match_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'keyword_plan_competition_level_pb2':"google.ads.google_ads.v4.proto.enums",
    'keyword_plan_forecast_interval_pb2':"google.ads.google_ads.v4.proto.enums",
    'keyword_plan_network_pb2':"google.ads.google_ads.v4.proto.enums",
    'label_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'legacy_app_install_ad_app_store_pb2':"google.ads.google_ads.v4.proto.enums",
    'linked_account_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'listing_group_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'local_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'location_extension_targeting_criterion_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'location_group_radius_units_pb2':"google.ads.google_ads.v4.proto.enums",
    'location_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'location_source_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'manager_link_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'matching_function_context_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'matching_function_operator_pb2':"google.ads.google_ads.v4.proto.enums",
    'media_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'merchant_center_link_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'message_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'mime_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'minute_of_hour_pb2':"google.ads.google_ads.v4.proto.enums",
    'mobile_app_vendor_pb2':"google.ads.google_ads.v4.proto.enums",
    'mobile_device_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'month_of_year_pb2':"google.ads.google_ads.v4.proto.enums",
    'negative_geo_target_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'offline_user_data_job_failure_reason_pb2':"google.ads.google_ads.v4.proto.enums",
    'offline_user_data_job_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'offline_user_data_job_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'operating_system_version_operator_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'optimization_goal_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'page_one_promoted_strategy_goal_pb2':"google.ads.google_ads.v4.proto.enums",
    'parental_status_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'payment_mode_pb2':"google.ads.google_ads.v4.proto.enums",
    'placeholder_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'placement_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'policy_approval_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'policy_review_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'policy_topic_entry_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'policy_topic_evidence_destination_mismatch_url_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'policy_topic_evidence_destination_not_working_device_pb2':"google.ads.google_ads.v4.proto.enums",
    'policy_topic_evidence_destination_not_working_dns_error_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'positive_geo_target_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'preferred_content_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'price_extension_price_qualifier_pb2':"google.ads.google_ads.v4.proto.enums",
    'price_extension_price_unit_pb2':"google.ads.google_ads.v4.proto.enums",
    'price_extension_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'price_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'product_bidding_category_level_pb2':"google.ads.google_ads.v4.proto.enums",
    'product_bidding_category_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'product_channel_exclusivity_pb2':"google.ads.google_ads.v4.proto.enums",
    'product_channel_pb2':"google.ads.google_ads.v4.proto.enums",
    'product_condition_pb2':"google.ads.google_ads.v4.proto.enums",
    'product_custom_attribute_index_pb2':"google.ads.google_ads.v4.proto.enums",
    'product_type_level_pb2':"google.ads.google_ads.v4.proto.enums",
    'promotion_extension_discount_modifier_pb2':"google.ads.google_ads.v4.proto.enums",
    'promotion_extension_occasion_pb2':"google.ads.google_ads.v4.proto.enums",
    'promotion_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'proximity_radius_units_pb2':"google.ads.google_ads.v4.proto.enums",
    'quality_score_bucket_pb2':"google.ads.google_ads.v4.proto.enums",
    'reach_plan_ad_length_pb2':"google.ads.google_ads.v4.proto.enums",
    'reach_plan_age_range_pb2':"google.ads.google_ads.v4.proto.enums",
    'reach_plan_network_pb2':"google.ads.google_ads.v4.proto.enums",
    'real_estate_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'recommendation_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'search_engine_results_page_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'search_term_match_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'search_term_targeting_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'served_asset_field_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'shared_set_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'shared_set_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'simulation_modification_method_pb2':"google.ads.google_ads.v4.proto.enums",
    'simulation_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'sitelink_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'slot_pb2':"google.ads.google_ads.v4.proto.enums",
    'spending_limit_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'structured_snippet_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'summary_row_setting_pb2':"google.ads.google_ads.v4.proto.enums",
    'system_managed_entity_source_pb2':"google.ads.google_ads.v4.proto.enums",
    'target_cpa_opt_in_recommendation_goal_pb2':"google.ads.google_ads.v4.proto.enums",
    'target_impression_share_location_pb2':"google.ads.google_ads.v4.proto.enums",
    'targeting_dimension_pb2':"google.ads.google_ads.v4.proto.enums",
    'time_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'tracking_code_page_format_pb2':"google.ads.google_ads.v4.proto.enums",
    'tracking_code_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'travel_placeholder_field_pb2':"google.ads.google_ads.v4.proto.enums",
    'user_interest_taxonomy_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'user_list_access_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'user_list_closing_reason_pb2':"google.ads.google_ads.v4.proto.enums",
    'user_list_combined_rule_operator_pb2':"google.ads.google_ads.v4.proto.enums",
    'user_list_crm_data_source_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'user_list_date_rule_item_operator_pb2':"google.ads.google_ads.v4.proto.enums",
    'user_list_logical_rule_operator_pb2':"google.ads.google_ads.v4.proto.enums",
    'user_list_membership_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'user_list_number_rule_item_operator_pb2':"google.ads.google_ads.v4.proto.enums",
    'user_list_prepopulation_status_pb2':"google.ads.google_ads.v4.proto.enums",
    'user_list_rule_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'user_list_size_range_pb2':"google.ads.google_ads.v4.proto.enums",
    'user_list_string_rule_item_operator_pb2':"google.ads.google_ads.v4.proto.enums",
    'user_list_type_pb2':"google.ads.google_ads.v4.proto.enums",
    'vanity_pharma_display_url_mode_pb2':"google.ads.google_ads.v4.proto.enums",
    'vanity_pharma_text_pb2':"google.ads.google_ads.v4.proto.enums",
    'webpage_condition_operand_pb2':"google.ads.google_ads.v4.proto.enums",
    'webpage_condition_operator_pb2':"google.ads.google_ads.v4.proto.enums",
    'access_invitation_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'account_budget_proposal_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'account_link_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'ad_customizer_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'ad_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'ad_group_ad_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'ad_group_bid_modifier_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'ad_group_criterion_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'ad_group_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'ad_group_feed_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'ad_parameter_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'ad_sharing_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'adx_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'asset_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'asset_link_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'authentication_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'authorization_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'batch_job_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'bidding_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'bidding_strategy_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'billing_setup_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'campaign_budget_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'campaign_criterion_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'campaign_draft_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'campaign_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'campaign_experiment_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'campaign_feed_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'campaign_shared_set_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'change_status_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'collection_size_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'context_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'conversion_action_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'conversion_adjustment_upload_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'conversion_upload_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'country_code_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'criterion_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'currency_code_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'custom_interest_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'customer_client_link_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'customer_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'customer_feed_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'customer_manager_link_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'database_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'date_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'date_range_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'distinct_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'enum_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'errors_pb2':"google.ads.google_ads.v4.proto.errors",
    'extension_feed_item_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'extension_setting_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'feed_attribute_reference_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'feed_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'feed_item_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'feed_item_target_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'feed_item_validation_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'feed_mapping_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'field_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'field_mask_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'function_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'function_parsing_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'geo_target_constant_suggestion_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'header_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'id_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'image_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'internal_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'invoice_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'keyword_plan_ad_group_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'keyword_plan_ad_group_keyword_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'keyword_plan_campaign_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'keyword_plan_campaign_keyword_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'keyword_plan_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'keyword_plan_idea_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'label_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'language_code_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'list_operation_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'manager_link_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'media_bundle_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'media_file_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'media_upload_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'multiplier_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'mutate_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'new_resource_creation_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'not_empty_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'not_whitelisted_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'null_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'offline_user_data_job_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'operation_access_denied_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'operator_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'partial_failure_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'payments_account_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'policy_finding_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'policy_validation_parameter_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'policy_violation_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'query_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'quota_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'range_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'reach_plan_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'recommendation_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'region_code_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'request_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'resource_access_denied_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'resource_count_limit_exceeded_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'setting_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'shared_criterion_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'shared_set_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'size_limit_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'string_format_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'string_length_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'third_party_app_analytics_link_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'time_zone_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'url_field_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'user_data_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'user_list_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'youtube_video_registration_error_pb2':"google.ads.google_ads.v4.proto.errors",
    'account_budget_pb2':"google.ads.google_ads.v4.proto.resources",
    'account_budget_proposal_pb2':"google.ads.google_ads.v4.proto.resources",
    'account_link_pb2':"google.ads.google_ads.v4.proto.resources",
    'ad_group_ad_asset_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'ad_group_ad_label_pb2':"google.ads.google_ads.v4.proto.resources",
    'ad_group_ad_pb2':"google.ads.google_ads.v4.proto.resources",
    'ad_group_audience_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'ad_group_bid_modifier_pb2':"google.ads.google_ads.v4.proto.resources",
    'ad_group_criterion_label_pb2':"google.ads.google_ads.v4.proto.resources",
    'ad_group_criterion_pb2':"google.ads.google_ads.v4.proto.resources",
    'ad_group_criterion_simulation_pb2':"google.ads.google_ads.v4.proto.resources",
    'ad_group_extension_setting_pb2':"google.ads.google_ads.v4.proto.resources",
    'ad_group_feed_pb2':"google.ads.google_ads.v4.proto.resources",
    'ad_group_label_pb2':"google.ads.google_ads.v4.proto.resources",
    'ad_group_pb2':"google.ads.google_ads.v4.proto.resources",
    'ad_group_simulation_pb2':"google.ads.google_ads.v4.proto.resources",
    'ad_parameter_pb2':"google.ads.google_ads.v4.proto.resources",
    'ad_pb2':"google.ads.google_ads.v4.proto.resources",
    'ad_schedule_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'age_range_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'asset_pb2':"google.ads.google_ads.v4.proto.resources",
    'batch_job_pb2':"google.ads.google_ads.v4.proto.resources",
    'bidding_strategy_pb2':"google.ads.google_ads.v4.proto.resources",
    'billing_setup_pb2':"google.ads.google_ads.v4.proto.resources",
    'campaign_audience_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'campaign_bid_modifier_pb2':"google.ads.google_ads.v4.proto.resources",
    'campaign_budget_pb2':"google.ads.google_ads.v4.proto.resources",
    'campaign_criterion_pb2':"google.ads.google_ads.v4.proto.resources",
    'campaign_criterion_simulation_pb2':"google.ads.google_ads.v4.proto.resources",
    'campaign_draft_pb2':"google.ads.google_ads.v4.proto.resources",
    'campaign_experiment_pb2':"google.ads.google_ads.v4.proto.resources",
    'campaign_extension_setting_pb2':"google.ads.google_ads.v4.proto.resources",
    'campaign_feed_pb2':"google.ads.google_ads.v4.proto.resources",
    'campaign_label_pb2':"google.ads.google_ads.v4.proto.resources",
    'campaign_pb2':"google.ads.google_ads.v4.proto.resources",
    'campaign_shared_set_pb2':"google.ads.google_ads.v4.proto.resources",
    'carrier_constant_pb2':"google.ads.google_ads.v4.proto.resources",
    'change_status_pb2':"google.ads.google_ads.v4.proto.resources",
    'click_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'conversion_action_pb2':"google.ads.google_ads.v4.proto.resources",
    'currency_constant_pb2':"google.ads.google_ads.v4.proto.resources",
    'custom_interest_pb2':"google.ads.google_ads.v4.proto.resources",
    'customer_client_link_pb2':"google.ads.google_ads.v4.proto.resources",
    'customer_client_pb2':"google.ads.google_ads.v4.proto.resources",
    'customer_extension_setting_pb2':"google.ads.google_ads.v4.proto.resources",
    'customer_feed_pb2':"google.ads.google_ads.v4.proto.resources",
    'customer_label_pb2':"google.ads.google_ads.v4.proto.resources",
    'customer_manager_link_pb2':"google.ads.google_ads.v4.proto.resources",
    'customer_negative_criterion_pb2':"google.ads.google_ads.v4.proto.resources",
    'customer_pb2':"google.ads.google_ads.v4.proto.resources",
    'detail_placement_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'display_keyword_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'distance_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'domain_category_pb2':"google.ads.google_ads.v4.proto.resources",
    'dynamic_search_ads_search_term_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'expanded_landing_page_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'extension_feed_item_pb2':"google.ads.google_ads.v4.proto.resources",
    'feed_item_pb2':"google.ads.google_ads.v4.proto.resources",
    'feed_item_target_pb2':"google.ads.google_ads.v4.proto.resources",
    'feed_mapping_pb2':"google.ads.google_ads.v4.proto.resources",
    'feed_pb2':"google.ads.google_ads.v4.proto.resources",
    'feed_placeholder_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'gender_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'geo_target_constant_pb2':"google.ads.google_ads.v4.proto.resources",
    'geographic_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'google_ads_field_pb2':"google.ads.google_ads.v4.proto.resources",
    'group_placement_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'hotel_group_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'hotel_performance_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'income_range_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'invoice_pb2':"google.ads.google_ads.v4.proto.resources",
    'keyword_plan_ad_group_keyword_pb2':"google.ads.google_ads.v4.proto.resources",
    'keyword_plan_ad_group_pb2':"google.ads.google_ads.v4.proto.resources",
    'keyword_plan_campaign_keyword_pb2':"google.ads.google_ads.v4.proto.resources",
    'keyword_plan_campaign_pb2':"google.ads.google_ads.v4.proto.resources",
    'keyword_plan_pb2':"google.ads.google_ads.v4.proto.resources",
    'keyword_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'label_pb2':"google.ads.google_ads.v4.proto.resources",
    'landing_page_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'language_constant_pb2':"google.ads.google_ads.v4.proto.resources",
    'location_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'managed_placement_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'media_file_pb2':"google.ads.google_ads.v4.proto.resources",
    'merchant_center_link_pb2':"google.ads.google_ads.v4.proto.resources",
    'mobile_app_category_constant_pb2':"google.ads.google_ads.v4.proto.resources",
    'mobile_device_constant_pb2':"google.ads.google_ads.v4.proto.resources",
    'offline_user_data_job_pb2':"google.ads.google_ads.v4.proto.resources",
    'operating_system_version_constant_pb2':"google.ads.google_ads.v4.proto.resources",
    'paid_organic_search_term_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'parental_status_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'payments_account_pb2':"google.ads.google_ads.v4.proto.resources",
    'product_bidding_category_constant_pb2':"google.ads.google_ads.v4.proto.resources",
    'product_group_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'recommendation_pb2':"google.ads.google_ads.v4.proto.resources",
    'remarketing_action_pb2':"google.ads.google_ads.v4.proto.resources",
    'search_term_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'shared_criterion_pb2':"google.ads.google_ads.v4.proto.resources",
    'shared_set_pb2':"google.ads.google_ads.v4.proto.resources",
    'shopping_performance_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'third_party_app_analytics_link_pb2':"google.ads.google_ads.v4.proto.resources",
    'topic_constant_pb2':"google.ads.google_ads.v4.proto.resources",
    'topic_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'user_interest_pb2':"google.ads.google_ads.v4.proto.resources",
    'user_list_pb2':"google.ads.google_ads.v4.proto.resources",
    'user_location_view_pb2':"google.ads.google_ads.v4.proto.resources",
    'video_pb2':"google.ads.google_ads.v4.proto.resources",
    'account_budget_proposal_service_pb2':"google.ads.google_ads.v4.proto.services",
    'account_budget_service_pb2':"google.ads.google_ads.v4.proto.services",
    'account_link_service_pb2':"google.ads.google_ads.v4.proto.services",
    'ad_group_ad_asset_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'ad_group_ad_label_service_pb2':"google.ads.google_ads.v4.proto.services",
    'ad_group_ad_service_pb2':"google.ads.google_ads.v4.proto.services",
    'ad_group_audience_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'ad_group_bid_modifier_service_pb2':"google.ads.google_ads.v4.proto.services",
    'ad_group_criterion_label_service_pb2':"google.ads.google_ads.v4.proto.services",
    'ad_group_criterion_service_pb2':"google.ads.google_ads.v4.proto.services",
    'ad_group_criterion_simulation_service_pb2':"google.ads.google_ads.v4.proto.services",
    'ad_group_extension_setting_service_pb2':"google.ads.google_ads.v4.proto.services",
    'ad_group_feed_service_pb2':"google.ads.google_ads.v4.proto.services",
    'ad_group_label_service_pb2':"google.ads.google_ads.v4.proto.services",
    'ad_group_service_pb2':"google.ads.google_ads.v4.proto.services",
    'ad_group_simulation_service_pb2':"google.ads.google_ads.v4.proto.services",
    'ad_parameter_service_pb2':"google.ads.google_ads.v4.proto.services",
    'ad_schedule_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'ad_service_pb2':"google.ads.google_ads.v4.proto.services",
    'age_range_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'asset_service_pb2':"google.ads.google_ads.v4.proto.services",
    'batch_job_service_pb2':"google.ads.google_ads.v4.proto.services",
    'bidding_strategy_service_pb2':"google.ads.google_ads.v4.proto.services",
    'billing_setup_service_pb2':"google.ads.google_ads.v4.proto.services",
    'campaign_audience_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'campaign_bid_modifier_service_pb2':"google.ads.google_ads.v4.proto.services",
    'campaign_budget_service_pb2':"google.ads.google_ads.v4.proto.services",
    'campaign_criterion_service_pb2':"google.ads.google_ads.v4.proto.services",
    'campaign_criterion_simulation_service_pb2':"google.ads.google_ads.v4.proto.services",
    'campaign_draft_service_pb2':"google.ads.google_ads.v4.proto.services",
    'campaign_experiment_service_pb2':"google.ads.google_ads.v4.proto.services",
    'campaign_extension_setting_service_pb2':"google.ads.google_ads.v4.proto.services",
    'campaign_feed_service_pb2':"google.ads.google_ads.v4.proto.services",
    'campaign_label_service_pb2':"google.ads.google_ads.v4.proto.services",
    'campaign_service_pb2':"google.ads.google_ads.v4.proto.services",
    'campaign_shared_set_service_pb2':"google.ads.google_ads.v4.proto.services",
    'carrier_constant_service_pb2':"google.ads.google_ads.v4.proto.services",
    'change_status_service_pb2':"google.ads.google_ads.v4.proto.services",
    'click_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'conversion_action_service_pb2':"google.ads.google_ads.v4.proto.services",
    'conversion_adjustment_upload_service_pb2':"google.ads.google_ads.v4.proto.services",
    'conversion_upload_service_pb2':"google.ads.google_ads.v4.proto.services",
    'currency_constant_service_pb2':"google.ads.google_ads.v4.proto.services",
    'custom_interest_service_pb2':"google.ads.google_ads.v4.proto.services",
    'customer_client_link_service_pb2':"google.ads.google_ads.v4.proto.services",
    'customer_client_service_pb2':"google.ads.google_ads.v4.proto.services",
    'customer_extension_setting_service_pb2':"google.ads.google_ads.v4.proto.services",
    'customer_feed_service_pb2':"google.ads.google_ads.v4.proto.services",
    'customer_label_service_pb2':"google.ads.google_ads.v4.proto.services",
    'customer_manager_link_service_pb2':"google.ads.google_ads.v4.proto.services",
    'customer_negative_criterion_service_pb2':"google.ads.google_ads.v4.proto.services",
    'customer_service_pb2':"google.ads.google_ads.v4.proto.services",
    'detail_placement_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'display_keyword_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'distance_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'domain_category_service_pb2':"google.ads.google_ads.v4.proto.services",
    'dynamic_search_ads_search_term_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'expanded_landing_page_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'extension_feed_item_service_pb2':"google.ads.google_ads.v4.proto.services",
    'feed_item_service_pb2':"google.ads.google_ads.v4.proto.services",
    'feed_item_target_service_pb2':"google.ads.google_ads.v4.proto.services",
    'feed_mapping_service_pb2':"google.ads.google_ads.v4.proto.services",
    'feed_placeholder_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'feed_service_pb2':"google.ads.google_ads.v4.proto.services",
    'gender_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'geo_target_constant_service_pb2':"google.ads.google_ads.v4.proto.services",
    'geographic_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'google_ads_field_service_pb2':"google.ads.google_ads.v4.proto.services",
    'google_ads_service_pb2':"google.ads.google_ads.v4.proto.services",
    'group_placement_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'hotel_group_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'hotel_performance_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'income_range_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'invoice_service_pb2':"google.ads.google_ads.v4.proto.services",
    'keyword_plan_ad_group_keyword_service_pb2':"google.ads.google_ads.v4.proto.services",
    'keyword_plan_ad_group_service_pb2':"google.ads.google_ads.v4.proto.services",
    'keyword_plan_campaign_keyword_service_pb2':"google.ads.google_ads.v4.proto.services",
    'keyword_plan_campaign_service_pb2':"google.ads.google_ads.v4.proto.services",
    'keyword_plan_idea_service_pb2':"google.ads.google_ads.v4.proto.services",
    'keyword_plan_service_pb2':"google.ads.google_ads.v4.proto.services",
    'keyword_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'label_service_pb2':"google.ads.google_ads.v4.proto.services",
    'landing_page_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'language_constant_service_pb2':"google.ads.google_ads.v4.proto.services",
    'location_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'managed_placement_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'media_file_service_pb2':"google.ads.google_ads.v4.proto.services",
    'merchant_center_link_service_pb2':"google.ads.google_ads.v4.proto.services",
    'mobile_app_category_constant_service_pb2':"google.ads.google_ads.v4.proto.services",
    'mobile_device_constant_service_pb2':"google.ads.google_ads.v4.proto.services",
    'offline_user_data_job_service_pb2':"google.ads.google_ads.v4.proto.services",
    'operating_system_version_constant_service_pb2':"google.ads.google_ads.v4.proto.services",
    'paid_organic_search_term_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'parental_status_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'payments_account_service_pb2':"google.ads.google_ads.v4.proto.services",
    'product_bidding_category_constant_service_pb2':"google.ads.google_ads.v4.proto.services",
    'product_group_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'reach_plan_service_pb2':"google.ads.google_ads.v4.proto.services",
    'recommendation_service_pb2':"google.ads.google_ads.v4.proto.services",
    'remarketing_action_service_pb2':"google.ads.google_ads.v4.proto.services",
    'search_term_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'shared_criterion_service_pb2':"google.ads.google_ads.v4.proto.services",
    'shared_set_service_pb2':"google.ads.google_ads.v4.proto.services",
    'shopping_performance_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'third_party_app_analytics_link_service_pb2':"google.ads.google_ads.v4.proto.services",
    'topic_constant_service_pb2':"google.ads.google_ads.v4.proto.services",
    'topic_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'user_data_service_pb2':"google.ads.google_ads.v4.proto.services",
    'user_interest_service_pb2':"google.ads.google_ads.v4.proto.services",
    'user_list_service_pb2':"google.ads.google_ads.v4.proto.services",
    'user_location_view_service_pb2':"google.ads.google_ads.v4.proto.services",
    'video_service_pb2':"google.ads.google_ads.v4.proto.services",
    'operations_pb2':"google.longrunning",
    'any_pb2':"google.protobuf",
    'empty_pb2':"google.protobuf",
    'field_mask_pb2':"google.protobuf",
    'wrappers_pb2':"google.protobuf",
    'status_pb2':"google.rpc",
}


_lazy_class_to_package_map = {
    'AccessInvitationErrorEnum':"google.ads.google_ads.v4.proto.errors.access_invitation_error_pb2",
    'AccessReasonEnum':"google.ads.google_ads.v4.proto.enums.access_reason_pb2",
    'AccessRoleEnum':"google.ads.google_ads.v4.proto.enums.access_role_pb2",
    'AccountBudget':"google.ads.google_ads.v4.proto.resources.account_budget_pb2",
    'AccountBudgetProposal':"google.ads.google_ads.v4.proto.resources.account_budget_proposal_pb2",
    'AccountBudgetProposalErrorEnum':"google.ads.google_ads.v4.proto.errors.account_budget_proposal_error_pb2",
    'AccountBudgetProposalOperation':"google.ads.google_ads.v4.proto.services.account_budget_proposal_service_pb2",
    'AccountBudgetProposalStatusEnum':"google.ads.google_ads.v4.proto.enums.account_budget_proposal_status_pb2",
    'AccountBudgetProposalTypeEnum':"google.ads.google_ads.v4.proto.enums.account_budget_proposal_type_pb2",
    'AccountBudgetStatusEnum':"google.ads.google_ads.v4.proto.enums.account_budget_status_pb2",
    'AccountLink':"google.ads.google_ads.v4.proto.resources.account_link_pb2",
    'AccountLinkOperation':"google.ads.google_ads.v4.proto.services.account_link_service_pb2",
    'AccountLinkStatusEnum':"google.ads.google_ads.v4.proto.enums.account_link_status_pb2",
    'Ad':"google.ads.google_ads.v4.proto.resources.ad_pb2",
    'AdCustomizerErrorEnum':"google.ads.google_ads.v4.proto.errors.ad_customizer_error_pb2",
    'AdCustomizerPlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.ad_customizer_placeholder_field_pb2",
    'AdErrorEnum':"google.ads.google_ads.v4.proto.errors.ad_error_pb2",
    'AdGroup':"google.ads.google_ads.v4.proto.resources.ad_group_pb2",
    'AdGroupAd':"google.ads.google_ads.v4.proto.resources.ad_group_ad_pb2",
    'AdGroupAdAssetPolicySummary':"google.ads.google_ads.v4.proto.resources.ad_group_ad_asset_view_pb2",
    'AdGroupAdAssetView':"google.ads.google_ads.v4.proto.resources.ad_group_ad_asset_view_pb2",
    'AdGroupAdErrorEnum':"google.ads.google_ads.v4.proto.errors.ad_group_ad_error_pb2",
    'AdGroupAdLabel':"google.ads.google_ads.v4.proto.resources.ad_group_ad_label_pb2",
    'AdGroupAdLabelOperation':"google.ads.google_ads.v4.proto.services.ad_group_ad_label_service_pb2",
    'AdGroupAdOperation':"google.ads.google_ads.v4.proto.services.ad_group_ad_service_pb2",
    'AdGroupAdPolicySummary':"google.ads.google_ads.v4.proto.resources.ad_group_ad_pb2",
    'AdGroupAdRotationModeEnum':"google.ads.google_ads.v4.proto.enums.ad_group_ad_rotation_mode_pb2",
    'AdGroupAdStatusEnum':"google.ads.google_ads.v4.proto.enums.ad_group_ad_status_pb2",
    'AdGroupAudienceView':"google.ads.google_ads.v4.proto.resources.ad_group_audience_view_pb2",
    'AdGroupBidModifier':"google.ads.google_ads.v4.proto.resources.ad_group_bid_modifier_pb2",
    'AdGroupBidModifierErrorEnum':"google.ads.google_ads.v4.proto.errors.ad_group_bid_modifier_error_pb2",
    'AdGroupBidModifierOperation':"google.ads.google_ads.v4.proto.services.ad_group_bid_modifier_service_pb2",
    'AdGroupCriterion':"google.ads.google_ads.v4.proto.resources.ad_group_criterion_pb2",
    'AdGroupCriterionApprovalStatusEnum':"google.ads.google_ads.v4.proto.enums.ad_group_criterion_approval_status_pb2",
    'AdGroupCriterionErrorEnum':"google.ads.google_ads.v4.proto.errors.ad_group_criterion_error_pb2",
    'AdGroupCriterionLabel':"google.ads.google_ads.v4.proto.resources.ad_group_criterion_label_pb2",
    'AdGroupCriterionLabelOperation':"google.ads.google_ads.v4.proto.services.ad_group_criterion_label_service_pb2",
    'AdGroupCriterionOperation':"google.ads.google_ads.v4.proto.services.ad_group_criterion_service_pb2",
    'AdGroupCriterionSimulation':"google.ads.google_ads.v4.proto.resources.ad_group_criterion_simulation_pb2",
    'AdGroupCriterionStatusEnum':"google.ads.google_ads.v4.proto.enums.ad_group_criterion_status_pb2",
    'AdGroupErrorEnum':"google.ads.google_ads.v4.proto.errors.ad_group_error_pb2",
    'AdGroupExtensionSetting':"google.ads.google_ads.v4.proto.resources.ad_group_extension_setting_pb2",
    'AdGroupExtensionSettingOperation':"google.ads.google_ads.v4.proto.services.ad_group_extension_setting_service_pb2",
    'AdGroupFeed':"google.ads.google_ads.v4.proto.resources.ad_group_feed_pb2",
    'AdGroupFeedErrorEnum':"google.ads.google_ads.v4.proto.errors.ad_group_feed_error_pb2",
    'AdGroupFeedOperation':"google.ads.google_ads.v4.proto.services.ad_group_feed_service_pb2",
    'AdGroupLabel':"google.ads.google_ads.v4.proto.resources.ad_group_label_pb2",
    'AdGroupLabelOperation':"google.ads.google_ads.v4.proto.services.ad_group_label_service_pb2",
    'AdGroupOperation':"google.ads.google_ads.v4.proto.services.ad_group_service_pb2",
    'AdGroupSimulation':"google.ads.google_ads.v4.proto.resources.ad_group_simulation_pb2",
    'AdGroupStatusEnum':"google.ads.google_ads.v4.proto.enums.ad_group_status_pb2",
    'AdGroupTypeEnum':"google.ads.google_ads.v4.proto.enums.ad_group_type_pb2",
    'AdImageAsset':"google.ads.google_ads.v4.proto.common.ad_asset_pb2",
    'AdMediaBundleAsset':"google.ads.google_ads.v4.proto.common.ad_asset_pb2",
    'AdNetworkTypeEnum':"google.ads.google_ads.v4.proto.enums.ad_network_type_pb2",
    'AdOperation':"google.ads.google_ads.v4.proto.services.ad_service_pb2",
    'AdParameter':"google.ads.google_ads.v4.proto.resources.ad_parameter_pb2",
    'AdParameterErrorEnum':"google.ads.google_ads.v4.proto.errors.ad_parameter_error_pb2",
    'AdParameterOperation':"google.ads.google_ads.v4.proto.services.ad_parameter_service_pb2",
    'AdScheduleInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'AdScheduleView':"google.ads.google_ads.v4.proto.resources.ad_schedule_view_pb2",
    'AdServingOptimizationStatusEnum':"google.ads.google_ads.v4.proto.enums.ad_serving_optimization_status_pb2",
    'AdSharingErrorEnum':"google.ads.google_ads.v4.proto.errors.ad_sharing_error_pb2",
    'AdStrengthEnum':"google.ads.google_ads.v4.proto.enums.ad_strength_pb2",
    'AdTextAsset':"google.ads.google_ads.v4.proto.common.ad_asset_pb2",
    'AdTypeEnum':"google.ads.google_ads.v4.proto.enums.ad_type_pb2",
    'AdVideoAsset':"google.ads.google_ads.v4.proto.common.ad_asset_pb2",
    'AddBatchJobOperationsRequest':"google.ads.google_ads.v4.proto.services.batch_job_service_pb2",
    'AddBatchJobOperationsResponse':"google.ads.google_ads.v4.proto.services.batch_job_service_pb2",
    'AddOfflineUserDataJobOperationsRequest':"google.ads.google_ads.v4.proto.services.offline_user_data_job_service_pb2",
    'AddOfflineUserDataJobOperationsResponse':"google.ads.google_ads.v4.proto.services.offline_user_data_job_service_pb2",
    'AddressInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'AdvertisingChannelSubTypeEnum':"google.ads.google_ads.v4.proto.enums.advertising_channel_sub_type_pb2",
    'AdvertisingChannelTypeEnum':"google.ads.google_ads.v4.proto.enums.advertising_channel_type_pb2",
    'AdxErrorEnum':"google.ads.google_ads.v4.proto.errors.adx_error_pb2",
    'AffiliateLocationFeedItem':"google.ads.google_ads.v4.proto.common.extensions_pb2",
    'AffiliateLocationFeedRelationshipTypeEnum':"google.ads.google_ads.v4.proto.enums.affiliate_location_feed_relationship_type_pb2",
    'AffiliateLocationPlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.affiliate_location_placeholder_field_pb2",
    'AgeRangeInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'AgeRangeTypeEnum':"google.ads.google_ads.v4.proto.enums.age_range_type_pb2",
    'AgeRangeView':"google.ads.google_ads.v4.proto.resources.age_range_view_pb2",
    'AppAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'AppCampaignAppStoreEnum':"google.ads.google_ads.v4.proto.enums.app_campaign_app_store_pb2",
    'AppCampaignBiddingStrategyGoalTypeEnum':"google.ads.google_ads.v4.proto.enums.app_campaign_bidding_strategy_goal_type_pb2",
    'AppEngagementAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'AppFeedItem':"google.ads.google_ads.v4.proto.common.extensions_pb2",
    'AppPaymentModelInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'AppPaymentModelTypeEnum':"google.ads.google_ads.v4.proto.enums.app_payment_model_type_pb2",
    'AppPlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.app_placeholder_field_pb2",
    'AppStoreEnum':"google.ads.google_ads.v4.proto.enums.app_store_pb2",
    'AppUrlOperatingSystemTypeEnum':"google.ads.google_ads.v4.proto.enums.app_url_operating_system_type_pb2",
    'ApplyRecommendationOperation':"google.ads.google_ads.v4.proto.services.recommendation_service_pb2",
    'ApplyRecommendationRequest':"google.ads.google_ads.v4.proto.services.recommendation_service_pb2",
    'ApplyRecommendationResponse':"google.ads.google_ads.v4.proto.services.recommendation_service_pb2",
    'ApplyRecommendationResult':"google.ads.google_ads.v4.proto.services.recommendation_service_pb2",
    'Asset':"google.ads.google_ads.v4.proto.resources.asset_pb2",
    'AssetErrorEnum':"google.ads.google_ads.v4.proto.errors.asset_error_pb2",
    'AssetFieldTypeEnum':"google.ads.google_ads.v4.proto.enums.asset_field_type_pb2",
    'AssetLinkErrorEnum':"google.ads.google_ads.v4.proto.errors.asset_link_error_pb2",
    'AssetOperation':"google.ads.google_ads.v4.proto.services.asset_service_pb2",
    'AssetPerformanceLabelEnum':"google.ads.google_ads.v4.proto.enums.asset_performance_label_pb2",
    'AssetTypeEnum':"google.ads.google_ads.v4.proto.enums.asset_type_pb2",
    'AttributeFieldMapping':"google.ads.google_ads.v4.proto.resources.feed_mapping_pb2",
    'AttributionModelEnum':"google.ads.google_ads.v4.proto.enums.attribution_model_pb2",
    'AuthenticationErrorEnum':"google.ads.google_ads.v4.proto.errors.authentication_error_pb2",
    'AuthorizationErrorEnum':"google.ads.google_ads.v4.proto.errors.authorization_error_pb2",
    'BasicUserListInfo':"google.ads.google_ads.v4.proto.common.user_lists_pb2",
    'BatchJob':"google.ads.google_ads.v4.proto.resources.batch_job_pb2",
    'BatchJobErrorEnum':"google.ads.google_ads.v4.proto.errors.batch_job_error_pb2",
    'BatchJobOperation':"google.ads.google_ads.v4.proto.services.batch_job_service_pb2",
    'BatchJobResult':"google.ads.google_ads.v4.proto.services.batch_job_service_pb2",
    'BatchJobStatusEnum':"google.ads.google_ads.v4.proto.enums.batch_job_status_pb2",
    'BidModifierSimulationPoint':"google.ads.google_ads.v4.proto.common.simulation_pb2",
    'BidModifierSimulationPointList':"google.ads.google_ads.v4.proto.common.simulation_pb2",
    'BidModifierSourceEnum':"google.ads.google_ads.v4.proto.enums.bid_modifier_source_pb2",
    'BiddingErrorEnum':"google.ads.google_ads.v4.proto.errors.bidding_error_pb2",
    'BiddingSourceEnum':"google.ads.google_ads.v4.proto.enums.bidding_source_pb2",
    'BiddingStrategy':"google.ads.google_ads.v4.proto.resources.bidding_strategy_pb2",
    'BiddingStrategyErrorEnum':"google.ads.google_ads.v4.proto.errors.bidding_strategy_error_pb2",
    'BiddingStrategyOperation':"google.ads.google_ads.v4.proto.services.bidding_strategy_service_pb2",
    'BiddingStrategyStatusEnum':"google.ads.google_ads.v4.proto.enums.bidding_strategy_status_pb2",
    'BiddingStrategyTypeEnum':"google.ads.google_ads.v4.proto.enums.bidding_strategy_type_pb2",
    'BillingSetup':"google.ads.google_ads.v4.proto.resources.billing_setup_pb2",
    'BillingSetupErrorEnum':"google.ads.google_ads.v4.proto.errors.billing_setup_error_pb2",
    'BillingSetupOperation':"google.ads.google_ads.v4.proto.services.billing_setup_service_pb2",
    'BillingSetupStatusEnum':"google.ads.google_ads.v4.proto.enums.billing_setup_status_pb2",
    'BookOnGoogleAsset':"google.ads.google_ads.v4.proto.common.asset_types_pb2",
    'BrandSafetySuitabilityEnum':"google.ads.google_ads.v4.proto.enums.brand_safety_suitability_pb2",
    'BudgetDeliveryMethodEnum':"google.ads.google_ads.v4.proto.enums.budget_delivery_method_pb2",
    'BudgetPeriodEnum':"google.ads.google_ads.v4.proto.enums.budget_period_pb2",
    'BudgetStatusEnum':"google.ads.google_ads.v4.proto.enums.budget_status_pb2",
    'BudgetTypeEnum':"google.ads.google_ads.v4.proto.enums.budget_type_pb2",
    'CallConversion':"google.ads.google_ads.v4.proto.services.conversion_upload_service_pb2",
    'CallConversionReportingStateEnum':"google.ads.google_ads.v4.proto.enums.call_conversion_reporting_state_pb2",
    'CallConversionResult':"google.ads.google_ads.v4.proto.services.conversion_upload_service_pb2",
    'CallFeedItem':"google.ads.google_ads.v4.proto.common.extensions_pb2",
    'CallOnlyAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'CallPlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.call_placeholder_field_pb2",
    'CallReportingSetting':"google.ads.google_ads.v4.proto.resources.customer_pb2",
    'CalloutFeedItem':"google.ads.google_ads.v4.proto.common.extensions_pb2",
    'CalloutPlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.callout_placeholder_field_pb2",
    'Campaign':"google.ads.google_ads.v4.proto.resources.campaign_pb2",
    'CampaignAudienceView':"google.ads.google_ads.v4.proto.resources.campaign_audience_view_pb2",
    'CampaignBidModifier':"google.ads.google_ads.v4.proto.resources.campaign_bid_modifier_pb2",
    'CampaignBidModifierOperation':"google.ads.google_ads.v4.proto.services.campaign_bid_modifier_service_pb2",
    'CampaignBudget':"google.ads.google_ads.v4.proto.resources.campaign_budget_pb2",
    'CampaignBudgetErrorEnum':"google.ads.google_ads.v4.proto.errors.campaign_budget_error_pb2",
    'CampaignBudgetOperation':"google.ads.google_ads.v4.proto.services.campaign_budget_service_pb2",
    'CampaignCriterion':"google.ads.google_ads.v4.proto.resources.campaign_criterion_pb2",
    'CampaignCriterionErrorEnum':"google.ads.google_ads.v4.proto.errors.campaign_criterion_error_pb2",
    'CampaignCriterionOperation':"google.ads.google_ads.v4.proto.services.campaign_criterion_service_pb2",
    'CampaignCriterionSimulation':"google.ads.google_ads.v4.proto.resources.campaign_criterion_simulation_pb2",
    'CampaignCriterionStatusEnum':"google.ads.google_ads.v4.proto.enums.campaign_criterion_status_pb2",
    'CampaignDraft':"google.ads.google_ads.v4.proto.resources.campaign_draft_pb2",
    'CampaignDraftErrorEnum':"google.ads.google_ads.v4.proto.errors.campaign_draft_error_pb2",
    'CampaignDraftOperation':"google.ads.google_ads.v4.proto.services.campaign_draft_service_pb2",
    'CampaignDraftStatusEnum':"google.ads.google_ads.v4.proto.enums.campaign_draft_status_pb2",
    'CampaignDuration':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'CampaignErrorEnum':"google.ads.google_ads.v4.proto.errors.campaign_error_pb2",
    'CampaignExperiment':"google.ads.google_ads.v4.proto.resources.campaign_experiment_pb2",
    'CampaignExperimentErrorEnum':"google.ads.google_ads.v4.proto.errors.campaign_experiment_error_pb2",
    'CampaignExperimentOperation':"google.ads.google_ads.v4.proto.services.campaign_experiment_service_pb2",
    'CampaignExperimentStatusEnum':"google.ads.google_ads.v4.proto.enums.campaign_experiment_status_pb2",
    'CampaignExperimentTrafficSplitTypeEnum':"google.ads.google_ads.v4.proto.enums.campaign_experiment_traffic_split_type_pb2",
    'CampaignExperimentTypeEnum':"google.ads.google_ads.v4.proto.enums.campaign_experiment_type_pb2",
    'CampaignExtensionSetting':"google.ads.google_ads.v4.proto.resources.campaign_extension_setting_pb2",
    'CampaignExtensionSettingOperation':"google.ads.google_ads.v4.proto.services.campaign_extension_setting_service_pb2",
    'CampaignFeed':"google.ads.google_ads.v4.proto.resources.campaign_feed_pb2",
    'CampaignFeedErrorEnum':"google.ads.google_ads.v4.proto.errors.campaign_feed_error_pb2",
    'CampaignFeedOperation':"google.ads.google_ads.v4.proto.services.campaign_feed_service_pb2",
    'CampaignLabel':"google.ads.google_ads.v4.proto.resources.campaign_label_pb2",
    'CampaignLabelOperation':"google.ads.google_ads.v4.proto.services.campaign_label_service_pb2",
    'CampaignOperation':"google.ads.google_ads.v4.proto.services.campaign_service_pb2",
    'CampaignServingStatusEnum':"google.ads.google_ads.v4.proto.enums.campaign_serving_status_pb2",
    'CampaignSharedSet':"google.ads.google_ads.v4.proto.resources.campaign_shared_set_pb2",
    'CampaignSharedSetErrorEnum':"google.ads.google_ads.v4.proto.errors.campaign_shared_set_error_pb2",
    'CampaignSharedSetOperation':"google.ads.google_ads.v4.proto.services.campaign_shared_set_service_pb2",
    'CampaignSharedSetStatusEnum':"google.ads.google_ads.v4.proto.enums.campaign_shared_set_status_pb2",
    'CampaignStatusEnum':"google.ads.google_ads.v4.proto.enums.campaign_status_pb2",
    'CarrierConstant':"google.ads.google_ads.v4.proto.resources.carrier_constant_pb2",
    'CarrierInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'ChangeStatus':"google.ads.google_ads.v4.proto.resources.change_status_pb2",
    'ChangeStatusErrorEnum':"google.ads.google_ads.v4.proto.errors.change_status_error_pb2",
    'ChangeStatusOperationEnum':"google.ads.google_ads.v4.proto.enums.change_status_operation_pb2",
    'ChangeStatusResourceTypeEnum':"google.ads.google_ads.v4.proto.enums.change_status_resource_type_pb2",
    'ClickConversion':"google.ads.google_ads.v4.proto.services.conversion_upload_service_pb2",
    'ClickConversionResult':"google.ads.google_ads.v4.proto.services.conversion_upload_service_pb2",
    'ClickLocation':"google.ads.google_ads.v4.proto.common.click_location_pb2",
    'ClickTypeEnum':"google.ads.google_ads.v4.proto.enums.click_type_pb2",
    'ClickView':"google.ads.google_ads.v4.proto.resources.click_view_pb2",
    'CollectionSizeErrorEnum':"google.ads.google_ads.v4.proto.errors.collection_size_error_pb2",
    'CombinedRuleUserListInfo':"google.ads.google_ads.v4.proto.common.user_lists_pb2",
    'Commission':"google.ads.google_ads.v4.proto.common.bidding_pb2",
    'ContentLabelInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'ContentLabelTypeEnum':"google.ads.google_ads.v4.proto.enums.content_label_type_pb2",
    'ContextErrorEnum':"google.ads.google_ads.v4.proto.errors.context_error_pb2",
    'ConversionAction':"google.ads.google_ads.v4.proto.resources.conversion_action_pb2",
    'ConversionActionCategoryEnum':"google.ads.google_ads.v4.proto.enums.conversion_action_category_pb2",
    'ConversionActionCountingTypeEnum':"google.ads.google_ads.v4.proto.enums.conversion_action_counting_type_pb2",
    'ConversionActionErrorEnum':"google.ads.google_ads.v4.proto.errors.conversion_action_error_pb2",
    'ConversionActionOperation':"google.ads.google_ads.v4.proto.services.conversion_action_service_pb2",
    'ConversionActionStatusEnum':"google.ads.google_ads.v4.proto.enums.conversion_action_status_pb2",
    'ConversionActionTypeEnum':"google.ads.google_ads.v4.proto.enums.conversion_action_type_pb2",
    'ConversionAdjustment':"google.ads.google_ads.v4.proto.services.conversion_adjustment_upload_service_pb2",
    'ConversionAdjustmentResult':"google.ads.google_ads.v4.proto.services.conversion_adjustment_upload_service_pb2",
    'ConversionAdjustmentTypeEnum':"google.ads.google_ads.v4.proto.enums.conversion_adjustment_type_pb2",
    'ConversionAdjustmentUploadErrorEnum':"google.ads.google_ads.v4.proto.errors.conversion_adjustment_upload_error_pb2",
    'ConversionAttributionEventTypeEnum':"google.ads.google_ads.v4.proto.enums.conversion_attribution_event_type_pb2",
    'ConversionLagBucketEnum':"google.ads.google_ads.v4.proto.enums.conversion_lag_bucket_pb2",
    'ConversionOrAdjustmentLagBucketEnum':"google.ads.google_ads.v4.proto.enums.conversion_or_adjustment_lag_bucket_pb2",
    'ConversionTrackingSetting':"google.ads.google_ads.v4.proto.resources.customer_pb2",
    'ConversionUploadErrorEnum':"google.ads.google_ads.v4.proto.errors.conversion_upload_error_pb2",
    'CountryCodeErrorEnum':"google.ads.google_ads.v4.proto.errors.country_code_error_pb2",
    'CpcBidSimulationPoint':"google.ads.google_ads.v4.proto.common.simulation_pb2",
    'CpcBidSimulationPointList':"google.ads.google_ads.v4.proto.common.simulation_pb2",
    'CpvBidSimulationPoint':"google.ads.google_ads.v4.proto.common.simulation_pb2",
    'CpvBidSimulationPointList':"google.ads.google_ads.v4.proto.common.simulation_pb2",
    'CreateCampaignExperimentMetadata':"google.ads.google_ads.v4.proto.services.campaign_experiment_service_pb2",
    'CreateCampaignExperimentRequest':"google.ads.google_ads.v4.proto.services.campaign_experiment_service_pb2",
    'CreateCustomerClientRequest':"google.ads.google_ads.v4.proto.services.customer_service_pb2",
    'CreateCustomerClientResponse':"google.ads.google_ads.v4.proto.services.customer_service_pb2",
    'CreateOfflineUserDataJobRequest':"google.ads.google_ads.v4.proto.services.offline_user_data_job_service_pb2",
    'CreateOfflineUserDataJobResponse':"google.ads.google_ads.v4.proto.services.offline_user_data_job_service_pb2",
    'CriterionCategoryAvailability':"google.ads.google_ads.v4.proto.common.criterion_category_availability_pb2",
    'CriterionCategoryChannelAvailability':"google.ads.google_ads.v4.proto.common.criterion_category_availability_pb2",
    'CriterionCategoryChannelAvailabilityModeEnum':"google.ads.google_ads.v4.proto.enums.criterion_category_channel_availability_mode_pb2",
    'CriterionCategoryLocaleAvailability':"google.ads.google_ads.v4.proto.common.criterion_category_availability_pb2",
    'CriterionCategoryLocaleAvailabilityModeEnum':"google.ads.google_ads.v4.proto.enums.criterion_category_locale_availability_mode_pb2",
    'CriterionErrorEnum':"google.ads.google_ads.v4.proto.errors.criterion_error_pb2",
    'CriterionSystemServingStatusEnum':"google.ads.google_ads.v4.proto.enums.criterion_system_serving_status_pb2",
    'CriterionTypeEnum':"google.ads.google_ads.v4.proto.enums.criterion_type_pb2",
    'CrmBasedUserListInfo':"google.ads.google_ads.v4.proto.common.user_lists_pb2",
    'CurrencyCodeErrorEnum':"google.ads.google_ads.v4.proto.errors.currency_code_error_pb2",
    'CurrencyConstant':"google.ads.google_ads.v4.proto.resources.currency_constant_pb2",
    'CustomAffinityInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'CustomIntentInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'CustomInterest':"google.ads.google_ads.v4.proto.resources.custom_interest_pb2",
    'CustomInterestErrorEnum':"google.ads.google_ads.v4.proto.errors.custom_interest_error_pb2",
    'CustomInterestMember':"google.ads.google_ads.v4.proto.resources.custom_interest_pb2",
    'CustomInterestMemberTypeEnum':"google.ads.google_ads.v4.proto.enums.custom_interest_member_type_pb2",
    'CustomInterestOperation':"google.ads.google_ads.v4.proto.services.custom_interest_service_pb2",
    'CustomInterestStatusEnum':"google.ads.google_ads.v4.proto.enums.custom_interest_status_pb2",
    'CustomInterestTypeEnum':"google.ads.google_ads.v4.proto.enums.custom_interest_type_pb2",
    'CustomParameter':"google.ads.google_ads.v4.proto.common.custom_parameter_pb2",
    'CustomPlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.custom_placeholder_field_pb2",
    'Customer':"google.ads.google_ads.v4.proto.resources.customer_pb2",
    'CustomerClient':"google.ads.google_ads.v4.proto.resources.customer_client_pb2",
    'CustomerClientLink':"google.ads.google_ads.v4.proto.resources.customer_client_link_pb2",
    'CustomerClientLinkErrorEnum':"google.ads.google_ads.v4.proto.errors.customer_client_link_error_pb2",
    'CustomerClientLinkOperation':"google.ads.google_ads.v4.proto.services.customer_client_link_service_pb2",
    'CustomerErrorEnum':"google.ads.google_ads.v4.proto.errors.customer_error_pb2",
    'CustomerExtensionSetting':"google.ads.google_ads.v4.proto.resources.customer_extension_setting_pb2",
    'CustomerExtensionSettingOperation':"google.ads.google_ads.v4.proto.services.customer_extension_setting_service_pb2",
    'CustomerFeed':"google.ads.google_ads.v4.proto.resources.customer_feed_pb2",
    'CustomerFeedErrorEnum':"google.ads.google_ads.v4.proto.errors.customer_feed_error_pb2",
    'CustomerFeedOperation':"google.ads.google_ads.v4.proto.services.customer_feed_service_pb2",
    'CustomerLabel':"google.ads.google_ads.v4.proto.resources.customer_label_pb2",
    'CustomerLabelOperation':"google.ads.google_ads.v4.proto.services.customer_label_service_pb2",
    'CustomerManagerLink':"google.ads.google_ads.v4.proto.resources.customer_manager_link_pb2",
    'CustomerManagerLinkErrorEnum':"google.ads.google_ads.v4.proto.errors.customer_manager_link_error_pb2",
    'CustomerManagerLinkOperation':"google.ads.google_ads.v4.proto.services.customer_manager_link_service_pb2",
    'CustomerMatchUploadKeyTypeEnum':"google.ads.google_ads.v4.proto.enums.customer_match_upload_key_type_pb2",
    'CustomerMatchUserListMetadata':"google.ads.google_ads.v4.proto.common.offline_user_data_pb2",
    'CustomerNegativeCriterion':"google.ads.google_ads.v4.proto.resources.customer_negative_criterion_pb2",
    'CustomerNegativeCriterionOperation':"google.ads.google_ads.v4.proto.services.customer_negative_criterion_service_pb2",
    'CustomerOperation':"google.ads.google_ads.v4.proto.services.customer_service_pb2",
    'CustomerPayPerConversionEligibilityFailureReasonEnum':"google.ads.google_ads.v4.proto.enums.customer_pay_per_conversion_eligibility_failure_reason_pb2",
    'DataDrivenModelStatusEnum':"google.ads.google_ads.v4.proto.enums.data_driven_model_status_pb2",
    'DatabaseErrorEnum':"google.ads.google_ads.v4.proto.errors.database_error_pb2",
    'DateErrorEnum':"google.ads.google_ads.v4.proto.errors.date_error_pb2",
    'DateRange':"google.ads.google_ads.v4.proto.common.dates_pb2",
    'DateRangeErrorEnum':"google.ads.google_ads.v4.proto.errors.date_range_error_pb2",
    'DateSpecificRuleUserListInfo':"google.ads.google_ads.v4.proto.common.user_lists_pb2",
    'DayOfWeekEnum':"google.ads.google_ads.v4.proto.enums.day_of_week_pb2",
    'DetailPlacementView':"google.ads.google_ads.v4.proto.resources.detail_placement_view_pb2",
    'DeviceEnum':"google.ads.google_ads.v4.proto.enums.device_pb2",
    'DeviceInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'DismissRecommendationRequest':"google.ads.google_ads.v4.proto.services.recommendation_service_pb2",
    'DismissRecommendationResponse':"google.ads.google_ads.v4.proto.services.recommendation_service_pb2",
    'DisplayAdFormatSettingEnum':"google.ads.google_ads.v4.proto.enums.display_ad_format_setting_pb2",
    'DisplayCallToAction':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'DisplayKeywordView':"google.ads.google_ads.v4.proto.resources.display_keyword_view_pb2",
    'DisplayUploadAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'DisplayUploadProductTypeEnum':"google.ads.google_ads.v4.proto.enums.display_upload_product_type_pb2",
    'DistanceBucketEnum':"google.ads.google_ads.v4.proto.enums.distance_bucket_pb2",
    'DistanceView':"google.ads.google_ads.v4.proto.resources.distance_view_pb2",
    'DistinctErrorEnum':"google.ads.google_ads.v4.proto.errors.distinct_error_pb2",
    'DomainCategory':"google.ads.google_ads.v4.proto.resources.domain_category_pb2",
    'DsaPageFeedCriterionFieldEnum':"google.ads.google_ads.v4.proto.enums.dsa_page_feed_criterion_field_pb2",
    'DynamicSearchAdsSearchTermView':"google.ads.google_ads.v4.proto.resources.dynamic_search_ads_search_term_view_pb2",
    'EducationPlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.education_placeholder_field_pb2",
    'EndCampaignExperimentRequest':"google.ads.google_ads.v4.proto.services.campaign_experiment_service_pb2",
    'EnhancedCpc':"google.ads.google_ads.v4.proto.common.bidding_pb2",
    'EnumErrorEnum':"google.ads.google_ads.v4.proto.errors.enum_error_pb2",
    'ErrorCode':"google.ads.google_ads.v4.proto.errors.errors_pb2",
    'ErrorDetails':"google.ads.google_ads.v4.proto.errors.errors_pb2",
    'ErrorLocation':"google.ads.google_ads.v4.proto.errors.errors_pb2",
    'ExpandedDynamicSearchAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'ExpandedLandingPageView':"google.ads.google_ads.v4.proto.resources.expanded_landing_page_view_pb2",
    'ExpandedTextAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'ExplorerAutoOptimizerSetting':"google.ads.google_ads.v4.proto.common.explorer_auto_optimizer_setting_pb2",
    'ExpressionRuleUserListInfo':"google.ads.google_ads.v4.proto.common.user_lists_pb2",
    'ExtensionFeedItem':"google.ads.google_ads.v4.proto.resources.extension_feed_item_pb2",
    'ExtensionFeedItemErrorEnum':"google.ads.google_ads.v4.proto.errors.extension_feed_item_error_pb2",
    'ExtensionFeedItemOperation':"google.ads.google_ads.v4.proto.services.extension_feed_item_service_pb2",
    'ExtensionSettingDeviceEnum':"google.ads.google_ads.v4.proto.enums.extension_setting_device_pb2",
    'ExtensionSettingErrorEnum':"google.ads.google_ads.v4.proto.errors.extension_setting_error_pb2",
    'ExtensionTypeEnum':"google.ads.google_ads.v4.proto.enums.extension_type_pb2",
    'ExternalAttributionData':"google.ads.google_ads.v4.proto.services.conversion_upload_service_pb2",
    'ExternalConversionSourceEnum':"google.ads.google_ads.v4.proto.enums.external_conversion_source_pb2",
    'Feed':"google.ads.google_ads.v4.proto.resources.feed_pb2",
    'FeedAttribute':"google.ads.google_ads.v4.proto.resources.feed_pb2",
    'FeedAttributeOperation':"google.ads.google_ads.v4.proto.resources.feed_pb2",
    'FeedAttributeReferenceErrorEnum':"google.ads.google_ads.v4.proto.errors.feed_attribute_reference_error_pb2",
    'FeedAttributeTypeEnum':"google.ads.google_ads.v4.proto.enums.feed_attribute_type_pb2",
    'FeedErrorEnum':"google.ads.google_ads.v4.proto.errors.feed_error_pb2",
    'FeedItem':"google.ads.google_ads.v4.proto.resources.feed_item_pb2",
    'FeedItemAttributeValue':"google.ads.google_ads.v4.proto.resources.feed_item_pb2",
    'FeedItemErrorEnum':"google.ads.google_ads.v4.proto.errors.feed_item_error_pb2",
    'FeedItemOperation':"google.ads.google_ads.v4.proto.services.feed_item_service_pb2",
    'FeedItemPlaceholderPolicyInfo':"google.ads.google_ads.v4.proto.resources.feed_item_pb2",
    'FeedItemQualityApprovalStatusEnum':"google.ads.google_ads.v4.proto.enums.feed_item_quality_approval_status_pb2",
    'FeedItemQualityDisapprovalReasonEnum':"google.ads.google_ads.v4.proto.enums.feed_item_quality_disapproval_reason_pb2",
    'FeedItemStatusEnum':"google.ads.google_ads.v4.proto.enums.feed_item_status_pb2",
    'FeedItemTarget':"google.ads.google_ads.v4.proto.resources.feed_item_target_pb2",
    'FeedItemTargetDeviceEnum':"google.ads.google_ads.v4.proto.enums.feed_item_target_device_pb2",
    'FeedItemTargetErrorEnum':"google.ads.google_ads.v4.proto.errors.feed_item_target_error_pb2",
    'FeedItemTargetOperation':"google.ads.google_ads.v4.proto.services.feed_item_target_service_pb2",
    'FeedItemTargetStatusEnum':"google.ads.google_ads.v4.proto.enums.feed_item_target_status_pb2",
    'FeedItemTargetTypeEnum':"google.ads.google_ads.v4.proto.enums.feed_item_target_type_pb2",
    'FeedItemValidationError':"google.ads.google_ads.v4.proto.resources.feed_item_pb2",
    'FeedItemValidationErrorEnum':"google.ads.google_ads.v4.proto.errors.feed_item_validation_error_pb2",
    'FeedItemValidationStatusEnum':"google.ads.google_ads.v4.proto.enums.feed_item_validation_status_pb2",
    'FeedLinkStatusEnum':"google.ads.google_ads.v4.proto.enums.feed_link_status_pb2",
    'FeedMapping':"google.ads.google_ads.v4.proto.resources.feed_mapping_pb2",
    'FeedMappingCriterionTypeEnum':"google.ads.google_ads.v4.proto.enums.feed_mapping_criterion_type_pb2",
    'FeedMappingErrorEnum':"google.ads.google_ads.v4.proto.errors.feed_mapping_error_pb2",
    'FeedMappingOperation':"google.ads.google_ads.v4.proto.services.feed_mapping_service_pb2",
    'FeedMappingStatusEnum':"google.ads.google_ads.v4.proto.enums.feed_mapping_status_pb2",
    'FeedOperation':"google.ads.google_ads.v4.proto.services.feed_service_pb2",
    'FeedOriginEnum':"google.ads.google_ads.v4.proto.enums.feed_origin_pb2",
    'FeedPlaceholderView':"google.ads.google_ads.v4.proto.resources.feed_placeholder_view_pb2",
    'FeedStatusEnum':"google.ads.google_ads.v4.proto.enums.feed_status_pb2",
    'FieldErrorEnum':"google.ads.google_ads.v4.proto.errors.field_error_pb2",
    'FieldMaskErrorEnum':"google.ads.google_ads.v4.proto.errors.field_mask_error_pb2",
    'FinalAppUrl':"google.ads.google_ads.v4.proto.common.final_app_url_pb2",
    'FlightPlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.flight_placeholder_field_pb2",
    'Forecast':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'ForecastMetrics':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'FrequencyCap':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'FrequencyCapEntry':"google.ads.google_ads.v4.proto.common.frequency_cap_pb2",
    'FrequencyCapEventTypeEnum':"google.ads.google_ads.v4.proto.enums.frequency_cap_event_type_pb2",
    'FrequencyCapKey':"google.ads.google_ads.v4.proto.common.frequency_cap_pb2",
    'FrequencyCapLevelEnum':"google.ads.google_ads.v4.proto.enums.frequency_cap_level_pb2",
    'FrequencyCapTimeUnitEnum':"google.ads.google_ads.v4.proto.enums.frequency_cap_time_unit_pb2",
    'FunctionErrorEnum':"google.ads.google_ads.v4.proto.errors.function_error_pb2",
    'FunctionParsingErrorEnum':"google.ads.google_ads.v4.proto.errors.function_parsing_error_pb2",
    'GclidDateTimePair':"google.ads.google_ads.v4.proto.services.conversion_adjustment_upload_service_pb2",
    'GenderInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'GenderTypeEnum':"google.ads.google_ads.v4.proto.enums.gender_type_pb2",
    'GenderView':"google.ads.google_ads.v4.proto.resources.gender_view_pb2",
    'GenerateForecastCurveRequest':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'GenerateForecastCurveResponse':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'GenerateForecastMetricsRequest':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'GenerateForecastMetricsResponse':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'GenerateHistoricalMetricsRequest':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'GenerateHistoricalMetricsResponse':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'GenerateKeywordIdeaResponse':"google.ads.google_ads.v4.proto.services.keyword_plan_idea_service_pb2",
    'GenerateKeywordIdeaResult':"google.ads.google_ads.v4.proto.services.keyword_plan_idea_service_pb2",
    'GenerateKeywordIdeasRequest':"google.ads.google_ads.v4.proto.services.keyword_plan_idea_service_pb2",
    'GenerateProductMixIdeasRequest':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'GenerateProductMixIdeasResponse':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'GenerateReachForecastRequest':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'GenerateReachForecastResponse':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'GeoPointInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'GeoTargetConstant':"google.ads.google_ads.v4.proto.resources.geo_target_constant_pb2",
    'GeoTargetConstantStatusEnum':"google.ads.google_ads.v4.proto.enums.geo_target_constant_status_pb2",
    'GeoTargetConstantSuggestion':"google.ads.google_ads.v4.proto.services.geo_target_constant_service_pb2",
    'GeoTargetConstantSuggestionErrorEnum':"google.ads.google_ads.v4.proto.errors.geo_target_constant_suggestion_error_pb2",
    'GeoTargetingRestrictionEnum':"google.ads.google_ads.v4.proto.enums.geo_targeting_restriction_pb2",
    'GeoTargetingTypeEnum':"google.ads.google_ads.v4.proto.enums.geo_targeting_type_pb2",
    'GeographicView':"google.ads.google_ads.v4.proto.resources.geographic_view_pb2",
    'GetAccountBudgetProposalRequest':"google.ads.google_ads.v4.proto.services.account_budget_proposal_service_pb2",
    'GetAccountBudgetRequest':"google.ads.google_ads.v4.proto.services.account_budget_service_pb2",
    'GetAccountLinkRequest':"google.ads.google_ads.v4.proto.services.account_link_service_pb2",
    'GetAdGroupAdAssetViewRequest':"google.ads.google_ads.v4.proto.services.ad_group_ad_asset_view_service_pb2",
    'GetAdGroupAdLabelRequest':"google.ads.google_ads.v4.proto.services.ad_group_ad_label_service_pb2",
    'GetAdGroupAdRequest':"google.ads.google_ads.v4.proto.services.ad_group_ad_service_pb2",
    'GetAdGroupAudienceViewRequest':"google.ads.google_ads.v4.proto.services.ad_group_audience_view_service_pb2",
    'GetAdGroupBidModifierRequest':"google.ads.google_ads.v4.proto.services.ad_group_bid_modifier_service_pb2",
    'GetAdGroupCriterionLabelRequest':"google.ads.google_ads.v4.proto.services.ad_group_criterion_label_service_pb2",
    'GetAdGroupCriterionRequest':"google.ads.google_ads.v4.proto.services.ad_group_criterion_service_pb2",
    'GetAdGroupCriterionSimulationRequest':"google.ads.google_ads.v4.proto.services.ad_group_criterion_simulation_service_pb2",
    'GetAdGroupExtensionSettingRequest':"google.ads.google_ads.v4.proto.services.ad_group_extension_setting_service_pb2",
    'GetAdGroupFeedRequest':"google.ads.google_ads.v4.proto.services.ad_group_feed_service_pb2",
    'GetAdGroupLabelRequest':"google.ads.google_ads.v4.proto.services.ad_group_label_service_pb2",
    'GetAdGroupRequest':"google.ads.google_ads.v4.proto.services.ad_group_service_pb2",
    'GetAdGroupSimulationRequest':"google.ads.google_ads.v4.proto.services.ad_group_simulation_service_pb2",
    'GetAdParameterRequest':"google.ads.google_ads.v4.proto.services.ad_parameter_service_pb2",
    'GetAdRequest':"google.ads.google_ads.v4.proto.services.ad_service_pb2",
    'GetAdScheduleViewRequest':"google.ads.google_ads.v4.proto.services.ad_schedule_view_service_pb2",
    'GetAgeRangeViewRequest':"google.ads.google_ads.v4.proto.services.age_range_view_service_pb2",
    'GetAssetRequest':"google.ads.google_ads.v4.proto.services.asset_service_pb2",
    'GetBatchJobRequest':"google.ads.google_ads.v4.proto.services.batch_job_service_pb2",
    'GetBiddingStrategyRequest':"google.ads.google_ads.v4.proto.services.bidding_strategy_service_pb2",
    'GetBillingSetupRequest':"google.ads.google_ads.v4.proto.services.billing_setup_service_pb2",
    'GetCampaignAudienceViewRequest':"google.ads.google_ads.v4.proto.services.campaign_audience_view_service_pb2",
    'GetCampaignBidModifierRequest':"google.ads.google_ads.v4.proto.services.campaign_bid_modifier_service_pb2",
    'GetCampaignBudgetRequest':"google.ads.google_ads.v4.proto.services.campaign_budget_service_pb2",
    'GetCampaignCriterionRequest':"google.ads.google_ads.v4.proto.services.campaign_criterion_service_pb2",
    'GetCampaignCriterionSimulationRequest':"google.ads.google_ads.v4.proto.services.campaign_criterion_simulation_service_pb2",
    'GetCampaignDraftRequest':"google.ads.google_ads.v4.proto.services.campaign_draft_service_pb2",
    'GetCampaignExperimentRequest':"google.ads.google_ads.v4.proto.services.campaign_experiment_service_pb2",
    'GetCampaignExtensionSettingRequest':"google.ads.google_ads.v4.proto.services.campaign_extension_setting_service_pb2",
    'GetCampaignFeedRequest':"google.ads.google_ads.v4.proto.services.campaign_feed_service_pb2",
    'GetCampaignLabelRequest':"google.ads.google_ads.v4.proto.services.campaign_label_service_pb2",
    'GetCampaignRequest':"google.ads.google_ads.v4.proto.services.campaign_service_pb2",
    'GetCampaignSharedSetRequest':"google.ads.google_ads.v4.proto.services.campaign_shared_set_service_pb2",
    'GetCarrierConstantRequest':"google.ads.google_ads.v4.proto.services.carrier_constant_service_pb2",
    'GetChangeStatusRequest':"google.ads.google_ads.v4.proto.services.change_status_service_pb2",
    'GetClickViewRequest':"google.ads.google_ads.v4.proto.services.click_view_service_pb2",
    'GetConversionActionRequest':"google.ads.google_ads.v4.proto.services.conversion_action_service_pb2",
    'GetCurrencyConstantRequest':"google.ads.google_ads.v4.proto.services.currency_constant_service_pb2",
    'GetCustomInterestRequest':"google.ads.google_ads.v4.proto.services.custom_interest_service_pb2",
    'GetCustomerClientLinkRequest':"google.ads.google_ads.v4.proto.services.customer_client_link_service_pb2",
    'GetCustomerClientRequest':"google.ads.google_ads.v4.proto.services.customer_client_service_pb2",
    'GetCustomerExtensionSettingRequest':"google.ads.google_ads.v4.proto.services.customer_extension_setting_service_pb2",
    'GetCustomerFeedRequest':"google.ads.google_ads.v4.proto.services.customer_feed_service_pb2",
    'GetCustomerLabelRequest':"google.ads.google_ads.v4.proto.services.customer_label_service_pb2",
    'GetCustomerManagerLinkRequest':"google.ads.google_ads.v4.proto.services.customer_manager_link_service_pb2",
    'GetCustomerNegativeCriterionRequest':"google.ads.google_ads.v4.proto.services.customer_negative_criterion_service_pb2",
    'GetCustomerRequest':"google.ads.google_ads.v4.proto.services.customer_service_pb2",
    'GetDetailPlacementViewRequest':"google.ads.google_ads.v4.proto.services.detail_placement_view_service_pb2",
    'GetDisplayKeywordViewRequest':"google.ads.google_ads.v4.proto.services.display_keyword_view_service_pb2",
    'GetDistanceViewRequest':"google.ads.google_ads.v4.proto.services.distance_view_service_pb2",
    'GetDomainCategoryRequest':"google.ads.google_ads.v4.proto.services.domain_category_service_pb2",
    'GetDynamicSearchAdsSearchTermViewRequest':"google.ads.google_ads.v4.proto.services.dynamic_search_ads_search_term_view_service_pb2",
    'GetExpandedLandingPageViewRequest':"google.ads.google_ads.v4.proto.services.expanded_landing_page_view_service_pb2",
    'GetExtensionFeedItemRequest':"google.ads.google_ads.v4.proto.services.extension_feed_item_service_pb2",
    'GetFeedItemRequest':"google.ads.google_ads.v4.proto.services.feed_item_service_pb2",
    'GetFeedItemTargetRequest':"google.ads.google_ads.v4.proto.services.feed_item_target_service_pb2",
    'GetFeedMappingRequest':"google.ads.google_ads.v4.proto.services.feed_mapping_service_pb2",
    'GetFeedPlaceholderViewRequest':"google.ads.google_ads.v4.proto.services.feed_placeholder_view_service_pb2",
    'GetFeedRequest':"google.ads.google_ads.v4.proto.services.feed_service_pb2",
    'GetGenderViewRequest':"google.ads.google_ads.v4.proto.services.gender_view_service_pb2",
    'GetGeoTargetConstantRequest':"google.ads.google_ads.v4.proto.services.geo_target_constant_service_pb2",
    'GetGeographicViewRequest':"google.ads.google_ads.v4.proto.services.geographic_view_service_pb2",
    'GetGoogleAdsFieldRequest':"google.ads.google_ads.v4.proto.services.google_ads_field_service_pb2",
    'GetGroupPlacementViewRequest':"google.ads.google_ads.v4.proto.services.group_placement_view_service_pb2",
    'GetHotelGroupViewRequest':"google.ads.google_ads.v4.proto.services.hotel_group_view_service_pb2",
    'GetHotelPerformanceViewRequest':"google.ads.google_ads.v4.proto.services.hotel_performance_view_service_pb2",
    'GetIncomeRangeViewRequest':"google.ads.google_ads.v4.proto.services.income_range_view_service_pb2",
    'GetKeywordPlanAdGroupKeywordRequest':"google.ads.google_ads.v4.proto.services.keyword_plan_ad_group_keyword_service_pb2",
    'GetKeywordPlanAdGroupRequest':"google.ads.google_ads.v4.proto.services.keyword_plan_ad_group_service_pb2",
    'GetKeywordPlanCampaignKeywordRequest':"google.ads.google_ads.v4.proto.services.keyword_plan_campaign_keyword_service_pb2",
    'GetKeywordPlanCampaignRequest':"google.ads.google_ads.v4.proto.services.keyword_plan_campaign_service_pb2",
    'GetKeywordPlanRequest':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'GetKeywordViewRequest':"google.ads.google_ads.v4.proto.services.keyword_view_service_pb2",
    'GetLabelRequest':"google.ads.google_ads.v4.proto.services.label_service_pb2",
    'GetLandingPageViewRequest':"google.ads.google_ads.v4.proto.services.landing_page_view_service_pb2",
    'GetLanguageConstantRequest':"google.ads.google_ads.v4.proto.services.language_constant_service_pb2",
    'GetLocationViewRequest':"google.ads.google_ads.v4.proto.services.location_view_service_pb2",
    'GetManagedPlacementViewRequest':"google.ads.google_ads.v4.proto.services.managed_placement_view_service_pb2",
    'GetMediaFileRequest':"google.ads.google_ads.v4.proto.services.media_file_service_pb2",
    'GetMerchantCenterLinkRequest':"google.ads.google_ads.v4.proto.services.merchant_center_link_service_pb2",
    'GetMobileAppCategoryConstantRequest':"google.ads.google_ads.v4.proto.services.mobile_app_category_constant_service_pb2",
    'GetMobileDeviceConstantRequest':"google.ads.google_ads.v4.proto.services.mobile_device_constant_service_pb2",
    'GetOfflineUserDataJobRequest':"google.ads.google_ads.v4.proto.services.offline_user_data_job_service_pb2",
    'GetOperatingSystemVersionConstantRequest':"google.ads.google_ads.v4.proto.services.operating_system_version_constant_service_pb2",
    'GetPaidOrganicSearchTermViewRequest':"google.ads.google_ads.v4.proto.services.paid_organic_search_term_view_service_pb2",
    'GetParentalStatusViewRequest':"google.ads.google_ads.v4.proto.services.parental_status_view_service_pb2",
    'GetProductBiddingCategoryConstantRequest':"google.ads.google_ads.v4.proto.services.product_bidding_category_constant_service_pb2",
    'GetProductGroupViewRequest':"google.ads.google_ads.v4.proto.services.product_group_view_service_pb2",
    'GetRecommendationRequest':"google.ads.google_ads.v4.proto.services.recommendation_service_pb2",
    'GetRemarketingActionRequest':"google.ads.google_ads.v4.proto.services.remarketing_action_service_pb2",
    'GetSearchTermViewRequest':"google.ads.google_ads.v4.proto.services.search_term_view_service_pb2",
    'GetSharedCriterionRequest':"google.ads.google_ads.v4.proto.services.shared_criterion_service_pb2",
    'GetSharedSetRequest':"google.ads.google_ads.v4.proto.services.shared_set_service_pb2",
    'GetShoppingPerformanceViewRequest':"google.ads.google_ads.v4.proto.services.shopping_performance_view_service_pb2",
    'GetThirdPartyAppAnalyticsLinkRequest':"google.ads.google_ads.v4.proto.services.third_party_app_analytics_link_service_pb2",
    'GetTopicConstantRequest':"google.ads.google_ads.v4.proto.services.topic_constant_service_pb2",
    'GetTopicViewRequest':"google.ads.google_ads.v4.proto.services.topic_view_service_pb2",
    'GetUserInterestRequest':"google.ads.google_ads.v4.proto.services.user_interest_service_pb2",
    'GetUserListRequest':"google.ads.google_ads.v4.proto.services.user_list_service_pb2",
    'GetUserLocationViewRequest':"google.ads.google_ads.v4.proto.services.user_location_view_service_pb2",
    'GetVideoRequest':"google.ads.google_ads.v4.proto.services.video_service_pb2",
    'GmailAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'GmailTeaser':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'GoogleAdsError':"google.ads.google_ads.v4.proto.errors.errors_pb2",
    'GoogleAdsFailure':"google.ads.google_ads.v4.proto.errors.errors_pb2",
    'GoogleAdsField':"google.ads.google_ads.v4.proto.resources.google_ads_field_pb2",
    'GoogleAdsFieldCategoryEnum':"google.ads.google_ads.v4.proto.enums.google_ads_field_category_pb2",
    'GoogleAdsFieldDataTypeEnum':"google.ads.google_ads.v4.proto.enums.google_ads_field_data_type_pb2",
    'GoogleAdsRow':"google.ads.google_ads.v4.proto.services.google_ads_service_pb2",
    'GraduateCampaignExperimentRequest':"google.ads.google_ads.v4.proto.services.campaign_experiment_service_pb2",
    'GraduateCampaignExperimentResponse':"google.ads.google_ads.v4.proto.services.campaign_experiment_service_pb2",
    'GroupPlacementView':"google.ads.google_ads.v4.proto.resources.group_placement_view_pb2",
    'HeaderErrorEnum':"google.ads.google_ads.v4.proto.errors.header_error_pb2",
    'HotelAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'HotelAdvanceBookingWindowInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'HotelCalloutFeedItem':"google.ads.google_ads.v4.proto.common.extensions_pb2",
    'HotelCheckInDayInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'HotelCityInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'HotelClassInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'HotelCountryRegionInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'HotelDateSelectionTypeEnum':"google.ads.google_ads.v4.proto.enums.hotel_date_selection_type_pb2",
    'HotelDateSelectionTypeInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'HotelGroupView':"google.ads.google_ads.v4.proto.resources.hotel_group_view_pb2",
    'HotelIdInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'HotelLengthOfStayInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'HotelPerformanceView':"google.ads.google_ads.v4.proto.resources.hotel_performance_view_pb2",
    'HotelPlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.hotel_placeholder_field_pb2",
    'HotelPriceBucketEnum':"google.ads.google_ads.v4.proto.enums.hotel_price_bucket_pb2",
    'HotelRateTypeEnum':"google.ads.google_ads.v4.proto.enums.hotel_rate_type_pb2",
    'HotelStateInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'IdErrorEnum':"google.ads.google_ads.v4.proto.errors.id_error_pb2",
    'ImageAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'ImageAsset':"google.ads.google_ads.v4.proto.common.asset_types_pb2",
    'ImageDimension':"google.ads.google_ads.v4.proto.common.asset_types_pb2",
    'ImageErrorEnum':"google.ads.google_ads.v4.proto.errors.image_error_pb2",
    'IncomeRangeInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'IncomeRangeTypeEnum':"google.ads.google_ads.v4.proto.enums.income_range_type_pb2",
    'IncomeRangeView':"google.ads.google_ads.v4.proto.resources.income_range_view_pb2",
    'InteractionEventTypeEnum':"google.ads.google_ads.v4.proto.enums.interaction_event_type_pb2",
    'InteractionTypeEnum':"google.ads.google_ads.v4.proto.enums.interaction_type_pb2",
    'InteractionTypeInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'InternalErrorEnum':"google.ads.google_ads.v4.proto.errors.internal_error_pb2",
    'Invoice':"google.ads.google_ads.v4.proto.resources.invoice_pb2",
    'InvoiceErrorEnum':"google.ads.google_ads.v4.proto.errors.invoice_error_pb2",
    'InvoiceTypeEnum':"google.ads.google_ads.v4.proto.enums.invoice_type_pb2",
    'IpBlockInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'JobPlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.job_placeholder_field_pb2",
    'Keyword':"google.ads.google_ads.v4.proto.common.segments_pb2",
    'KeywordAndUrlSeed':"google.ads.google_ads.v4.proto.services.keyword_plan_idea_service_pb2",
    'KeywordInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'KeywordMatchTypeEnum':"google.ads.google_ads.v4.proto.enums.keyword_match_type_pb2",
    'KeywordPlan':"google.ads.google_ads.v4.proto.resources.keyword_plan_pb2",
    'KeywordPlanAdGroup':"google.ads.google_ads.v4.proto.resources.keyword_plan_ad_group_pb2",
    'KeywordPlanAdGroupErrorEnum':"google.ads.google_ads.v4.proto.errors.keyword_plan_ad_group_error_pb2",
    'KeywordPlanAdGroupForecast':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'KeywordPlanAdGroupKeyword':"google.ads.google_ads.v4.proto.resources.keyword_plan_ad_group_keyword_pb2",
    'KeywordPlanAdGroupKeywordErrorEnum':"google.ads.google_ads.v4.proto.errors.keyword_plan_ad_group_keyword_error_pb2",
    'KeywordPlanAdGroupKeywordOperation':"google.ads.google_ads.v4.proto.services.keyword_plan_ad_group_keyword_service_pb2",
    'KeywordPlanAdGroupOperation':"google.ads.google_ads.v4.proto.services.keyword_plan_ad_group_service_pb2",
    'KeywordPlanCampaign':"google.ads.google_ads.v4.proto.resources.keyword_plan_campaign_pb2",
    'KeywordPlanCampaignErrorEnum':"google.ads.google_ads.v4.proto.errors.keyword_plan_campaign_error_pb2",
    'KeywordPlanCampaignForecast':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'KeywordPlanCampaignForecastCurve':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'KeywordPlanCampaignKeyword':"google.ads.google_ads.v4.proto.resources.keyword_plan_campaign_keyword_pb2",
    'KeywordPlanCampaignKeywordErrorEnum':"google.ads.google_ads.v4.proto.errors.keyword_plan_campaign_keyword_error_pb2",
    'KeywordPlanCampaignKeywordOperation':"google.ads.google_ads.v4.proto.services.keyword_plan_campaign_keyword_service_pb2",
    'KeywordPlanCampaignOperation':"google.ads.google_ads.v4.proto.services.keyword_plan_campaign_service_pb2",
    'KeywordPlanCompetitionLevelEnum':"google.ads.google_ads.v4.proto.enums.keyword_plan_competition_level_pb2",
    'KeywordPlanErrorEnum':"google.ads.google_ads.v4.proto.errors.keyword_plan_error_pb2",
    'KeywordPlanForecastIntervalEnum':"google.ads.google_ads.v4.proto.enums.keyword_plan_forecast_interval_pb2",
    'KeywordPlanForecastPeriod':"google.ads.google_ads.v4.proto.resources.keyword_plan_pb2",
    'KeywordPlanGeoTarget':"google.ads.google_ads.v4.proto.resources.keyword_plan_campaign_pb2",
    'KeywordPlanHistoricalMetrics':"google.ads.google_ads.v4.proto.common.keyword_plan_common_pb2",
    'KeywordPlanIdeaErrorEnum':"google.ads.google_ads.v4.proto.errors.keyword_plan_idea_error_pb2",
    'KeywordPlanKeywordForecast':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'KeywordPlanKeywordHistoricalMetrics':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'KeywordPlanMaxCpcBidForecast':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'KeywordPlanMaxCpcBidForecastCurve':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'KeywordPlanNetworkEnum':"google.ads.google_ads.v4.proto.enums.keyword_plan_network_pb2",
    'KeywordPlanOperation':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'KeywordSeed':"google.ads.google_ads.v4.proto.services.keyword_plan_idea_service_pb2",
    'KeywordView':"google.ads.google_ads.v4.proto.resources.keyword_view_pb2",
    'Label':"google.ads.google_ads.v4.proto.resources.label_pb2",
    'LabelErrorEnum':"google.ads.google_ads.v4.proto.errors.label_error_pb2",
    'LabelOperation':"google.ads.google_ads.v4.proto.services.label_service_pb2",
    'LabelStatusEnum':"google.ads.google_ads.v4.proto.enums.label_status_pb2",
    'LandingPageView':"google.ads.google_ads.v4.proto.resources.landing_page_view_pb2",
    'LanguageCodeErrorEnum':"google.ads.google_ads.v4.proto.errors.language_code_error_pb2",
    'LanguageConstant':"google.ads.google_ads.v4.proto.resources.language_constant_pb2",
    'LanguageInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'LegacyAppInstallAdAppStoreEnum':"google.ads.google_ads.v4.proto.enums.legacy_app_install_ad_app_store_pb2",
    'LegacyAppInstallAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'LegacyResponsiveDisplayAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'LinkedAccountTypeEnum':"google.ads.google_ads.v4.proto.enums.linked_account_type_pb2",
    'ListAccessibleCustomersRequest':"google.ads.google_ads.v4.proto.services.customer_service_pb2",
    'ListAccessibleCustomersResponse':"google.ads.google_ads.v4.proto.services.customer_service_pb2",
    'ListBatchJobResultsRequest':"google.ads.google_ads.v4.proto.services.batch_job_service_pb2",
    'ListBatchJobResultsResponse':"google.ads.google_ads.v4.proto.services.batch_job_service_pb2",
    'ListCampaignDraftAsyncErrorsRequest':"google.ads.google_ads.v4.proto.services.campaign_draft_service_pb2",
    'ListCampaignDraftAsyncErrorsResponse':"google.ads.google_ads.v4.proto.services.campaign_draft_service_pb2",
    'ListCampaignExperimentAsyncErrorsRequest':"google.ads.google_ads.v4.proto.services.campaign_experiment_service_pb2",
    'ListCampaignExperimentAsyncErrorsResponse':"google.ads.google_ads.v4.proto.services.campaign_experiment_service_pb2",
    'ListInvoicesRequest':"google.ads.google_ads.v4.proto.services.invoice_service_pb2",
    'ListInvoicesResponse':"google.ads.google_ads.v4.proto.services.invoice_service_pb2",
    'ListMerchantCenterLinksRequest':"google.ads.google_ads.v4.proto.services.merchant_center_link_service_pb2",
    'ListMerchantCenterLinksResponse':"google.ads.google_ads.v4.proto.services.merchant_center_link_service_pb2",
    'ListOperationErrorEnum':"google.ads.google_ads.v4.proto.errors.list_operation_error_pb2",
    'ListPaymentsAccountsRequest':"google.ads.google_ads.v4.proto.services.payments_account_service_pb2",
    'ListPaymentsAccountsResponse':"google.ads.google_ads.v4.proto.services.payments_account_service_pb2",
    'ListPlannableLocationsRequest':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'ListPlannableLocationsResponse':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'ListPlannableProductsRequest':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'ListPlannableProductsResponse':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'ListingDimensionInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'ListingGroupInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'ListingGroupTypeEnum':"google.ads.google_ads.v4.proto.enums.listing_group_type_pb2",
    'ListingScopeInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'LocalAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'LocalPlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.local_placeholder_field_pb2",
    'LocationExtensionTargetingCriterionFieldEnum':"google.ads.google_ads.v4.proto.enums.location_extension_targeting_criterion_field_pb2",
    'LocationFeedItem':"google.ads.google_ads.v4.proto.common.extensions_pb2",
    'LocationGroupInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'LocationGroupRadiusUnitsEnum':"google.ads.google_ads.v4.proto.enums.location_group_radius_units_pb2",
    'LocationInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'LocationPlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.location_placeholder_field_pb2",
    'LocationSourceTypeEnum':"google.ads.google_ads.v4.proto.enums.location_source_type_pb2",
    'LocationView':"google.ads.google_ads.v4.proto.resources.location_view_pb2",
    'LogicalUserListInfo':"google.ads.google_ads.v4.proto.common.user_lists_pb2",
    'LogicalUserListOperandInfo':"google.ads.google_ads.v4.proto.common.user_lists_pb2",
    'ManagedPlacementView':"google.ads.google_ads.v4.proto.resources.managed_placement_view_pb2",
    'ManagerLinkErrorEnum':"google.ads.google_ads.v4.proto.errors.manager_link_error_pb2",
    'ManagerLinkStatusEnum':"google.ads.google_ads.v4.proto.enums.manager_link_status_pb2",
    'ManualCpc':"google.ads.google_ads.v4.proto.common.bidding_pb2",
    'ManualCpm':"google.ads.google_ads.v4.proto.common.bidding_pb2",
    'ManualCpv':"google.ads.google_ads.v4.proto.common.bidding_pb2",
    'MatchingFunction':"google.ads.google_ads.v4.proto.common.matching_function_pb2",
    'MatchingFunctionContextTypeEnum':"google.ads.google_ads.v4.proto.enums.matching_function_context_type_pb2",
    'MatchingFunctionOperatorEnum':"google.ads.google_ads.v4.proto.enums.matching_function_operator_pb2",
    'MaximizeConversionValue':"google.ads.google_ads.v4.proto.common.bidding_pb2",
    'MaximizeConversions':"google.ads.google_ads.v4.proto.common.bidding_pb2",
    'MediaAudio':"google.ads.google_ads.v4.proto.resources.media_file_pb2",
    'MediaBundle':"google.ads.google_ads.v4.proto.resources.media_file_pb2",
    'MediaBundleAsset':"google.ads.google_ads.v4.proto.common.asset_types_pb2",
    'MediaBundleErrorEnum':"google.ads.google_ads.v4.proto.errors.media_bundle_error_pb2",
    'MediaFile':"google.ads.google_ads.v4.proto.resources.media_file_pb2",
    'MediaFileErrorEnum':"google.ads.google_ads.v4.proto.errors.media_file_error_pb2",
    'MediaFileOperation':"google.ads.google_ads.v4.proto.services.media_file_service_pb2",
    'MediaImage':"google.ads.google_ads.v4.proto.resources.media_file_pb2",
    'MediaTypeEnum':"google.ads.google_ads.v4.proto.enums.media_type_pb2",
    'MediaUploadErrorEnum':"google.ads.google_ads.v4.proto.errors.media_upload_error_pb2",
    'MediaVideo':"google.ads.google_ads.v4.proto.resources.media_file_pb2",
    'MerchantCenterLink':"google.ads.google_ads.v4.proto.resources.merchant_center_link_pb2",
    'MerchantCenterLinkOperation':"google.ads.google_ads.v4.proto.services.merchant_center_link_service_pb2",
    'MerchantCenterLinkStatusEnum':"google.ads.google_ads.v4.proto.enums.merchant_center_link_status_pb2",
    'MessagePlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.message_placeholder_field_pb2",
    'Metrics':"google.ads.google_ads.v4.proto.common.metrics_pb2",
    'MimeTypeEnum':"google.ads.google_ads.v4.proto.enums.mime_type_pb2",
    'MinuteOfHourEnum':"google.ads.google_ads.v4.proto.enums.minute_of_hour_pb2",
    'MobileAppCategoryConstant':"google.ads.google_ads.v4.proto.resources.mobile_app_category_constant_pb2",
    'MobileAppCategoryInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'MobileAppVendorEnum':"google.ads.google_ads.v4.proto.enums.mobile_app_vendor_pb2",
    'MobileApplicationInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'MobileDeviceConstant':"google.ads.google_ads.v4.proto.resources.mobile_device_constant_pb2",
    'MobileDeviceInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'MobileDeviceTypeEnum':"google.ads.google_ads.v4.proto.enums.mobile_device_type_pb2",
    'Money':"google.ads.google_ads.v4.proto.common.feed_common_pb2",
    'MonthOfYearEnum':"google.ads.google_ads.v4.proto.enums.month_of_year_pb2",
    'MonthlySearchVolume':"google.ads.google_ads.v4.proto.common.keyword_plan_common_pb2",
    'MoveManagerLinkRequest':"google.ads.google_ads.v4.proto.services.customer_manager_link_service_pb2",
    'MoveManagerLinkResponse':"google.ads.google_ads.v4.proto.services.customer_manager_link_service_pb2",
    'MultiplierErrorEnum':"google.ads.google_ads.v4.proto.errors.multiplier_error_pb2",
    'MutateAccountBudgetProposalRequest':"google.ads.google_ads.v4.proto.services.account_budget_proposal_service_pb2",
    'MutateAccountBudgetProposalResponse':"google.ads.google_ads.v4.proto.services.account_budget_proposal_service_pb2",
    'MutateAccountBudgetProposalResult':"google.ads.google_ads.v4.proto.services.account_budget_proposal_service_pb2",
    'MutateAccountLinkRequest':"google.ads.google_ads.v4.proto.services.account_link_service_pb2",
    'MutateAccountLinkResponse':"google.ads.google_ads.v4.proto.services.account_link_service_pb2",
    'MutateAccountLinkResult':"google.ads.google_ads.v4.proto.services.account_link_service_pb2",
    'MutateAdGroupAdLabelResult':"google.ads.google_ads.v4.proto.services.ad_group_ad_label_service_pb2",
    'MutateAdGroupAdLabelsRequest':"google.ads.google_ads.v4.proto.services.ad_group_ad_label_service_pb2",
    'MutateAdGroupAdLabelsResponse':"google.ads.google_ads.v4.proto.services.ad_group_ad_label_service_pb2",
    'MutateAdGroupAdResult':"google.ads.google_ads.v4.proto.services.ad_group_ad_service_pb2",
    'MutateAdGroupAdsRequest':"google.ads.google_ads.v4.proto.services.ad_group_ad_service_pb2",
    'MutateAdGroupAdsResponse':"google.ads.google_ads.v4.proto.services.ad_group_ad_service_pb2",
    'MutateAdGroupBidModifierResult':"google.ads.google_ads.v4.proto.services.ad_group_bid_modifier_service_pb2",
    'MutateAdGroupBidModifiersRequest':"google.ads.google_ads.v4.proto.services.ad_group_bid_modifier_service_pb2",
    'MutateAdGroupBidModifiersResponse':"google.ads.google_ads.v4.proto.services.ad_group_bid_modifier_service_pb2",
    'MutateAdGroupCriteriaRequest':"google.ads.google_ads.v4.proto.services.ad_group_criterion_service_pb2",
    'MutateAdGroupCriteriaResponse':"google.ads.google_ads.v4.proto.services.ad_group_criterion_service_pb2",
    'MutateAdGroupCriterionLabelResult':"google.ads.google_ads.v4.proto.services.ad_group_criterion_label_service_pb2",
    'MutateAdGroupCriterionLabelsRequest':"google.ads.google_ads.v4.proto.services.ad_group_criterion_label_service_pb2",
    'MutateAdGroupCriterionLabelsResponse':"google.ads.google_ads.v4.proto.services.ad_group_criterion_label_service_pb2",
    'MutateAdGroupCriterionResult':"google.ads.google_ads.v4.proto.services.ad_group_criterion_service_pb2",
    'MutateAdGroupExtensionSettingResult':"google.ads.google_ads.v4.proto.services.ad_group_extension_setting_service_pb2",
    'MutateAdGroupExtensionSettingsRequest':"google.ads.google_ads.v4.proto.services.ad_group_extension_setting_service_pb2",
    'MutateAdGroupExtensionSettingsResponse':"google.ads.google_ads.v4.proto.services.ad_group_extension_setting_service_pb2",
    'MutateAdGroupFeedResult':"google.ads.google_ads.v4.proto.services.ad_group_feed_service_pb2",
    'MutateAdGroupFeedsRequest':"google.ads.google_ads.v4.proto.services.ad_group_feed_service_pb2",
    'MutateAdGroupFeedsResponse':"google.ads.google_ads.v4.proto.services.ad_group_feed_service_pb2",
    'MutateAdGroupLabelResult':"google.ads.google_ads.v4.proto.services.ad_group_label_service_pb2",
    'MutateAdGroupLabelsRequest':"google.ads.google_ads.v4.proto.services.ad_group_label_service_pb2",
    'MutateAdGroupLabelsResponse':"google.ads.google_ads.v4.proto.services.ad_group_label_service_pb2",
    'MutateAdGroupResult':"google.ads.google_ads.v4.proto.services.ad_group_service_pb2",
    'MutateAdGroupsRequest':"google.ads.google_ads.v4.proto.services.ad_group_service_pb2",
    'MutateAdGroupsResponse':"google.ads.google_ads.v4.proto.services.ad_group_service_pb2",
    'MutateAdParameterResult':"google.ads.google_ads.v4.proto.services.ad_parameter_service_pb2",
    'MutateAdParametersRequest':"google.ads.google_ads.v4.proto.services.ad_parameter_service_pb2",
    'MutateAdParametersResponse':"google.ads.google_ads.v4.proto.services.ad_parameter_service_pb2",
    'MutateAdResult':"google.ads.google_ads.v4.proto.services.ad_service_pb2",
    'MutateAdsRequest':"google.ads.google_ads.v4.proto.services.ad_service_pb2",
    'MutateAdsResponse':"google.ads.google_ads.v4.proto.services.ad_service_pb2",
    'MutateAssetResult':"google.ads.google_ads.v4.proto.services.asset_service_pb2",
    'MutateAssetsRequest':"google.ads.google_ads.v4.proto.services.asset_service_pb2",
    'MutateAssetsResponse':"google.ads.google_ads.v4.proto.services.asset_service_pb2",
    'MutateBatchJobRequest':"google.ads.google_ads.v4.proto.services.batch_job_service_pb2",
    'MutateBatchJobResponse':"google.ads.google_ads.v4.proto.services.batch_job_service_pb2",
    'MutateBatchJobResult':"google.ads.google_ads.v4.proto.services.batch_job_service_pb2",
    'MutateBiddingStrategiesRequest':"google.ads.google_ads.v4.proto.services.bidding_strategy_service_pb2",
    'MutateBiddingStrategiesResponse':"google.ads.google_ads.v4.proto.services.bidding_strategy_service_pb2",
    'MutateBiddingStrategyResult':"google.ads.google_ads.v4.proto.services.bidding_strategy_service_pb2",
    'MutateBillingSetupRequest':"google.ads.google_ads.v4.proto.services.billing_setup_service_pb2",
    'MutateBillingSetupResponse':"google.ads.google_ads.v4.proto.services.billing_setup_service_pb2",
    'MutateBillingSetupResult':"google.ads.google_ads.v4.proto.services.billing_setup_service_pb2",
    'MutateCampaignBidModifierResult':"google.ads.google_ads.v4.proto.services.campaign_bid_modifier_service_pb2",
    'MutateCampaignBidModifiersRequest':"google.ads.google_ads.v4.proto.services.campaign_bid_modifier_service_pb2",
    'MutateCampaignBidModifiersResponse':"google.ads.google_ads.v4.proto.services.campaign_bid_modifier_service_pb2",
    'MutateCampaignBudgetResult':"google.ads.google_ads.v4.proto.services.campaign_budget_service_pb2",
    'MutateCampaignBudgetsRequest':"google.ads.google_ads.v4.proto.services.campaign_budget_service_pb2",
    'MutateCampaignBudgetsResponse':"google.ads.google_ads.v4.proto.services.campaign_budget_service_pb2",
    'MutateCampaignCriteriaRequest':"google.ads.google_ads.v4.proto.services.campaign_criterion_service_pb2",
    'MutateCampaignCriteriaResponse':"google.ads.google_ads.v4.proto.services.campaign_criterion_service_pb2",
    'MutateCampaignCriterionResult':"google.ads.google_ads.v4.proto.services.campaign_criterion_service_pb2",
    'MutateCampaignDraftResult':"google.ads.google_ads.v4.proto.services.campaign_draft_service_pb2",
    'MutateCampaignDraftsRequest':"google.ads.google_ads.v4.proto.services.campaign_draft_service_pb2",
    'MutateCampaignDraftsResponse':"google.ads.google_ads.v4.proto.services.campaign_draft_service_pb2",
    'MutateCampaignExperimentResult':"google.ads.google_ads.v4.proto.services.campaign_experiment_service_pb2",
    'MutateCampaignExperimentsRequest':"google.ads.google_ads.v4.proto.services.campaign_experiment_service_pb2",
    'MutateCampaignExperimentsResponse':"google.ads.google_ads.v4.proto.services.campaign_experiment_service_pb2",
    'MutateCampaignExtensionSettingResult':"google.ads.google_ads.v4.proto.services.campaign_extension_setting_service_pb2",
    'MutateCampaignExtensionSettingsRequest':"google.ads.google_ads.v4.proto.services.campaign_extension_setting_service_pb2",
    'MutateCampaignExtensionSettingsResponse':"google.ads.google_ads.v4.proto.services.campaign_extension_setting_service_pb2",
    'MutateCampaignFeedResult':"google.ads.google_ads.v4.proto.services.campaign_feed_service_pb2",
    'MutateCampaignFeedsRequest':"google.ads.google_ads.v4.proto.services.campaign_feed_service_pb2",
    'MutateCampaignFeedsResponse':"google.ads.google_ads.v4.proto.services.campaign_feed_service_pb2",
    'MutateCampaignLabelResult':"google.ads.google_ads.v4.proto.services.campaign_label_service_pb2",
    'MutateCampaignLabelsRequest':"google.ads.google_ads.v4.proto.services.campaign_label_service_pb2",
    'MutateCampaignLabelsResponse':"google.ads.google_ads.v4.proto.services.campaign_label_service_pb2",
    'MutateCampaignResult':"google.ads.google_ads.v4.proto.services.campaign_service_pb2",
    'MutateCampaignSharedSetResult':"google.ads.google_ads.v4.proto.services.campaign_shared_set_service_pb2",
    'MutateCampaignSharedSetsRequest':"google.ads.google_ads.v4.proto.services.campaign_shared_set_service_pb2",
    'MutateCampaignSharedSetsResponse':"google.ads.google_ads.v4.proto.services.campaign_shared_set_service_pb2",
    'MutateCampaignsRequest':"google.ads.google_ads.v4.proto.services.campaign_service_pb2",
    'MutateCampaignsResponse':"google.ads.google_ads.v4.proto.services.campaign_service_pb2",
    'MutateConversionActionResult':"google.ads.google_ads.v4.proto.services.conversion_action_service_pb2",
    'MutateConversionActionsRequest':"google.ads.google_ads.v4.proto.services.conversion_action_service_pb2",
    'MutateConversionActionsResponse':"google.ads.google_ads.v4.proto.services.conversion_action_service_pb2",
    'MutateCustomInterestResult':"google.ads.google_ads.v4.proto.services.custom_interest_service_pb2",
    'MutateCustomInterestsRequest':"google.ads.google_ads.v4.proto.services.custom_interest_service_pb2",
    'MutateCustomInterestsResponse':"google.ads.google_ads.v4.proto.services.custom_interest_service_pb2",
    'MutateCustomerClientLinkRequest':"google.ads.google_ads.v4.proto.services.customer_client_link_service_pb2",
    'MutateCustomerClientLinkResponse':"google.ads.google_ads.v4.proto.services.customer_client_link_service_pb2",
    'MutateCustomerClientLinkResult':"google.ads.google_ads.v4.proto.services.customer_client_link_service_pb2",
    'MutateCustomerExtensionSettingResult':"google.ads.google_ads.v4.proto.services.customer_extension_setting_service_pb2",
    'MutateCustomerExtensionSettingsRequest':"google.ads.google_ads.v4.proto.services.customer_extension_setting_service_pb2",
    'MutateCustomerExtensionSettingsResponse':"google.ads.google_ads.v4.proto.services.customer_extension_setting_service_pb2",
    'MutateCustomerFeedResult':"google.ads.google_ads.v4.proto.services.customer_feed_service_pb2",
    'MutateCustomerFeedsRequest':"google.ads.google_ads.v4.proto.services.customer_feed_service_pb2",
    'MutateCustomerFeedsResponse':"google.ads.google_ads.v4.proto.services.customer_feed_service_pb2",
    'MutateCustomerLabelResult':"google.ads.google_ads.v4.proto.services.customer_label_service_pb2",
    'MutateCustomerLabelsRequest':"google.ads.google_ads.v4.proto.services.customer_label_service_pb2",
    'MutateCustomerLabelsResponse':"google.ads.google_ads.v4.proto.services.customer_label_service_pb2",
    'MutateCustomerManagerLinkRequest':"google.ads.google_ads.v4.proto.services.customer_manager_link_service_pb2",
    'MutateCustomerManagerLinkResponse':"google.ads.google_ads.v4.proto.services.customer_manager_link_service_pb2",
    'MutateCustomerManagerLinkResult':"google.ads.google_ads.v4.proto.services.customer_manager_link_service_pb2",
    'MutateCustomerNegativeCriteriaRequest':"google.ads.google_ads.v4.proto.services.customer_negative_criterion_service_pb2",
    'MutateCustomerNegativeCriteriaResponse':"google.ads.google_ads.v4.proto.services.customer_negative_criterion_service_pb2",
    'MutateCustomerNegativeCriteriaResult':"google.ads.google_ads.v4.proto.services.customer_negative_criterion_service_pb2",
    'MutateCustomerRequest':"google.ads.google_ads.v4.proto.services.customer_service_pb2",
    'MutateCustomerResponse':"google.ads.google_ads.v4.proto.services.customer_service_pb2",
    'MutateCustomerResult':"google.ads.google_ads.v4.proto.services.customer_service_pb2",
    'MutateErrorEnum':"google.ads.google_ads.v4.proto.errors.mutate_error_pb2",
    'MutateExtensionFeedItemResult':"google.ads.google_ads.v4.proto.services.extension_feed_item_service_pb2",
    'MutateExtensionFeedItemsRequest':"google.ads.google_ads.v4.proto.services.extension_feed_item_service_pb2",
    'MutateExtensionFeedItemsResponse':"google.ads.google_ads.v4.proto.services.extension_feed_item_service_pb2",
    'MutateFeedItemResult':"google.ads.google_ads.v4.proto.services.feed_item_service_pb2",
    'MutateFeedItemTargetResult':"google.ads.google_ads.v4.proto.services.feed_item_target_service_pb2",
    'MutateFeedItemTargetsRequest':"google.ads.google_ads.v4.proto.services.feed_item_target_service_pb2",
    'MutateFeedItemTargetsResponse':"google.ads.google_ads.v4.proto.services.feed_item_target_service_pb2",
    'MutateFeedItemsRequest':"google.ads.google_ads.v4.proto.services.feed_item_service_pb2",
    'MutateFeedItemsResponse':"google.ads.google_ads.v4.proto.services.feed_item_service_pb2",
    'MutateFeedMappingResult':"google.ads.google_ads.v4.proto.services.feed_mapping_service_pb2",
    'MutateFeedMappingsRequest':"google.ads.google_ads.v4.proto.services.feed_mapping_service_pb2",
    'MutateFeedMappingsResponse':"google.ads.google_ads.v4.proto.services.feed_mapping_service_pb2",
    'MutateFeedResult':"google.ads.google_ads.v4.proto.services.feed_service_pb2",
    'MutateFeedsRequest':"google.ads.google_ads.v4.proto.services.feed_service_pb2",
    'MutateFeedsResponse':"google.ads.google_ads.v4.proto.services.feed_service_pb2",
    'MutateGoogleAdsRequest':"google.ads.google_ads.v4.proto.services.google_ads_service_pb2",
    'MutateGoogleAdsResponse':"google.ads.google_ads.v4.proto.services.google_ads_service_pb2",
    'MutateKeywordPlanAdGroupKeywordResult':"google.ads.google_ads.v4.proto.services.keyword_plan_ad_group_keyword_service_pb2",
    'MutateKeywordPlanAdGroupKeywordsRequest':"google.ads.google_ads.v4.proto.services.keyword_plan_ad_group_keyword_service_pb2",
    'MutateKeywordPlanAdGroupKeywordsResponse':"google.ads.google_ads.v4.proto.services.keyword_plan_ad_group_keyword_service_pb2",
    'MutateKeywordPlanAdGroupResult':"google.ads.google_ads.v4.proto.services.keyword_plan_ad_group_service_pb2",
    'MutateKeywordPlanAdGroupsRequest':"google.ads.google_ads.v4.proto.services.keyword_plan_ad_group_service_pb2",
    'MutateKeywordPlanAdGroupsResponse':"google.ads.google_ads.v4.proto.services.keyword_plan_ad_group_service_pb2",
    'MutateKeywordPlanCampaignKeywordResult':"google.ads.google_ads.v4.proto.services.keyword_plan_campaign_keyword_service_pb2",
    'MutateKeywordPlanCampaignKeywordsRequest':"google.ads.google_ads.v4.proto.services.keyword_plan_campaign_keyword_service_pb2",
    'MutateKeywordPlanCampaignKeywordsResponse':"google.ads.google_ads.v4.proto.services.keyword_plan_campaign_keyword_service_pb2",
    'MutateKeywordPlanCampaignResult':"google.ads.google_ads.v4.proto.services.keyword_plan_campaign_service_pb2",
    'MutateKeywordPlanCampaignsRequest':"google.ads.google_ads.v4.proto.services.keyword_plan_campaign_service_pb2",
    'MutateKeywordPlanCampaignsResponse':"google.ads.google_ads.v4.proto.services.keyword_plan_campaign_service_pb2",
    'MutateKeywordPlansRequest':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'MutateKeywordPlansResponse':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'MutateKeywordPlansResult':"google.ads.google_ads.v4.proto.services.keyword_plan_service_pb2",
    'MutateLabelResult':"google.ads.google_ads.v4.proto.services.label_service_pb2",
    'MutateLabelsRequest':"google.ads.google_ads.v4.proto.services.label_service_pb2",
    'MutateLabelsResponse':"google.ads.google_ads.v4.proto.services.label_service_pb2",
    'MutateMediaFileResult':"google.ads.google_ads.v4.proto.services.media_file_service_pb2",
    'MutateMediaFilesRequest':"google.ads.google_ads.v4.proto.services.media_file_service_pb2",
    'MutateMediaFilesResponse':"google.ads.google_ads.v4.proto.services.media_file_service_pb2",
    'MutateMerchantCenterLinkRequest':"google.ads.google_ads.v4.proto.services.merchant_center_link_service_pb2",
    'MutateMerchantCenterLinkResponse':"google.ads.google_ads.v4.proto.services.merchant_center_link_service_pb2",
    'MutateMerchantCenterLinkResult':"google.ads.google_ads.v4.proto.services.merchant_center_link_service_pb2",
    'MutateOperation':"google.ads.google_ads.v4.proto.services.google_ads_service_pb2",
    'MutateOperationResponse':"google.ads.google_ads.v4.proto.services.google_ads_service_pb2",
    'MutateRemarketingActionResult':"google.ads.google_ads.v4.proto.services.remarketing_action_service_pb2",
    'MutateRemarketingActionsRequest':"google.ads.google_ads.v4.proto.services.remarketing_action_service_pb2",
    'MutateRemarketingActionsResponse':"google.ads.google_ads.v4.proto.services.remarketing_action_service_pb2",
    'MutateSharedCriteriaRequest':"google.ads.google_ads.v4.proto.services.shared_criterion_service_pb2",
    'MutateSharedCriteriaResponse':"google.ads.google_ads.v4.proto.services.shared_criterion_service_pb2",
    'MutateSharedCriterionResult':"google.ads.google_ads.v4.proto.services.shared_criterion_service_pb2",
    'MutateSharedSetResult':"google.ads.google_ads.v4.proto.services.shared_set_service_pb2",
    'MutateSharedSetsRequest':"google.ads.google_ads.v4.proto.services.shared_set_service_pb2",
    'MutateSharedSetsResponse':"google.ads.google_ads.v4.proto.services.shared_set_service_pb2",
    'MutateUserListResult':"google.ads.google_ads.v4.proto.services.user_list_service_pb2",
    'MutateUserListsRequest':"google.ads.google_ads.v4.proto.services.user_list_service_pb2",
    'MutateUserListsResponse':"google.ads.google_ads.v4.proto.services.user_list_service_pb2",
    'NegativeGeoTargetTypeEnum':"google.ads.google_ads.v4.proto.enums.negative_geo_target_type_pb2",
    'NewResourceCreationErrorEnum':"google.ads.google_ads.v4.proto.errors.new_resource_creation_error_pb2",
    'NotEmptyErrorEnum':"google.ads.google_ads.v4.proto.errors.not_empty_error_pb2",
    'NotWhitelistedErrorEnum':"google.ads.google_ads.v4.proto.errors.not_whitelisted_error_pb2",
    'NullErrorEnum':"google.ads.google_ads.v4.proto.errors.null_error_pb2",
    'OfflineUserAddressInfo':"google.ads.google_ads.v4.proto.common.offline_user_data_pb2",
    'OfflineUserDataJob':"google.ads.google_ads.v4.proto.resources.offline_user_data_job_pb2",
    'OfflineUserDataJobErrorEnum':"google.ads.google_ads.v4.proto.errors.offline_user_data_job_error_pb2",
    'OfflineUserDataJobFailureReasonEnum':"google.ads.google_ads.v4.proto.enums.offline_user_data_job_failure_reason_pb2",
    'OfflineUserDataJobOperation':"google.ads.google_ads.v4.proto.services.offline_user_data_job_service_pb2",
    'OfflineUserDataJobStatusEnum':"google.ads.google_ads.v4.proto.enums.offline_user_data_job_status_pb2",
    'OfflineUserDataJobTypeEnum':"google.ads.google_ads.v4.proto.enums.offline_user_data_job_type_pb2",
    'OnTargetAudienceMetrics':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'Operand':"google.ads.google_ads.v4.proto.common.matching_function_pb2",
    'OperatingSystemVersionConstant':"google.ads.google_ads.v4.proto.resources.operating_system_version_constant_pb2",
    'OperatingSystemVersionInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'OperatingSystemVersionOperatorTypeEnum':"google.ads.google_ads.v4.proto.enums.operating_system_version_operator_type_pb2",
    'OperationAccessDeniedErrorEnum':"google.ads.google_ads.v4.proto.errors.operation_access_denied_error_pb2",
    'OperatorErrorEnum':"google.ads.google_ads.v4.proto.errors.operator_error_pb2",
    'OptimizationGoalTypeEnum':"google.ads.google_ads.v4.proto.enums.optimization_goal_type_pb2",
    'PaidOrganicSearchTermView':"google.ads.google_ads.v4.proto.resources.paid_organic_search_term_view_pb2",
    'ParentalStatusInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'ParentalStatusTypeEnum':"google.ads.google_ads.v4.proto.enums.parental_status_type_pb2",
    'ParentalStatusView':"google.ads.google_ads.v4.proto.resources.parental_status_view_pb2",
    'PartialFailureErrorEnum':"google.ads.google_ads.v4.proto.errors.partial_failure_error_pb2",
    'PaymentModeEnum':"google.ads.google_ads.v4.proto.enums.payment_mode_pb2",
    'PaymentsAccount':"google.ads.google_ads.v4.proto.resources.payments_account_pb2",
    'PaymentsAccountErrorEnum':"google.ads.google_ads.v4.proto.errors.payments_account_error_pb2",
    'PercentCpc':"google.ads.google_ads.v4.proto.common.bidding_pb2",
    'PlaceholderTypeEnum':"google.ads.google_ads.v4.proto.enums.placeholder_type_pb2",
    'PlacementInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'PlacementTypeEnum':"google.ads.google_ads.v4.proto.enums.placement_type_pb2",
    'PlannableLocation':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'PlannableTargeting':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'PlannedProduct':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'PolicyApprovalStatusEnum':"google.ads.google_ads.v4.proto.enums.policy_approval_status_pb2",
    'PolicyFindingDetails':"google.ads.google_ads.v4.proto.errors.errors_pb2",
    'PolicyFindingErrorEnum':"google.ads.google_ads.v4.proto.errors.policy_finding_error_pb2",
    'PolicyReviewStatusEnum':"google.ads.google_ads.v4.proto.enums.policy_review_status_pb2",
    'PolicyTopicConstraint':"google.ads.google_ads.v4.proto.common.policy_pb2",
    'PolicyTopicEntry':"google.ads.google_ads.v4.proto.common.policy_pb2",
    'PolicyTopicEntryTypeEnum':"google.ads.google_ads.v4.proto.enums.policy_topic_entry_type_pb2",
    'PolicyTopicEvidence':"google.ads.google_ads.v4.proto.common.policy_pb2",
    'PolicyTopicEvidenceDestinationMismatchUrlTypeEnum':"google.ads.google_ads.v4.proto.enums.policy_topic_evidence_destination_mismatch_url_type_pb2",
    'PolicyTopicEvidenceDestinationNotWorkingDeviceEnum':"google.ads.google_ads.v4.proto.enums.policy_topic_evidence_destination_not_working_device_pb2",
    'PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum':"google.ads.google_ads.v4.proto.enums.policy_topic_evidence_destination_not_working_dns_error_type_pb2",
    'PolicyValidationParameter':"google.ads.google_ads.v4.proto.common.policy_pb2",
    'PolicyValidationParameterErrorEnum':"google.ads.google_ads.v4.proto.errors.policy_validation_parameter_error_pb2",
    'PolicyViolationDetails':"google.ads.google_ads.v4.proto.errors.errors_pb2",
    'PolicyViolationErrorEnum':"google.ads.google_ads.v4.proto.errors.policy_violation_error_pb2",
    'PolicyViolationKey':"google.ads.google_ads.v4.proto.common.policy_pb2",
    'PositiveGeoTargetTypeEnum':"google.ads.google_ads.v4.proto.enums.positive_geo_target_type_pb2",
    'Preferences':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'PreferredContentInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'PreferredContentTypeEnum':"google.ads.google_ads.v4.proto.enums.preferred_content_type_pb2",
    'PriceExtensionPriceQualifierEnum':"google.ads.google_ads.v4.proto.enums.price_extension_price_qualifier_pb2",
    'PriceExtensionPriceUnitEnum':"google.ads.google_ads.v4.proto.enums.price_extension_price_unit_pb2",
    'PriceExtensionTypeEnum':"google.ads.google_ads.v4.proto.enums.price_extension_type_pb2",
    'PriceFeedItem':"google.ads.google_ads.v4.proto.common.extensions_pb2",
    'PriceOffer':"google.ads.google_ads.v4.proto.common.extensions_pb2",
    'PricePlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.price_placeholder_field_pb2",
    'ProductAllocation':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'ProductBiddingCategoryConstant':"google.ads.google_ads.v4.proto.resources.product_bidding_category_constant_pb2",
    'ProductBiddingCategoryInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'ProductBiddingCategoryLevelEnum':"google.ads.google_ads.v4.proto.enums.product_bidding_category_level_pb2",
    'ProductBiddingCategoryStatusEnum':"google.ads.google_ads.v4.proto.enums.product_bidding_category_status_pb2",
    'ProductBrandInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'ProductChannelEnum':"google.ads.google_ads.v4.proto.enums.product_channel_pb2",
    'ProductChannelExclusivityEnum':"google.ads.google_ads.v4.proto.enums.product_channel_exclusivity_pb2",
    'ProductChannelExclusivityInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'ProductChannelInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'ProductConditionEnum':"google.ads.google_ads.v4.proto.enums.product_condition_pb2",
    'ProductConditionInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'ProductCustomAttributeIndexEnum':"google.ads.google_ads.v4.proto.enums.product_custom_attribute_index_pb2",
    'ProductCustomAttributeInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'ProductGroupView':"google.ads.google_ads.v4.proto.resources.product_group_view_pb2",
    'ProductImage':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'ProductItemIdInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'ProductMetadata':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'ProductTypeInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'ProductTypeLevelEnum':"google.ads.google_ads.v4.proto.enums.product_type_level_pb2",
    'ProductVideo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'PromoteCampaignDraftRequest':"google.ads.google_ads.v4.proto.services.campaign_draft_service_pb2",
    'PromoteCampaignExperimentRequest':"google.ads.google_ads.v4.proto.services.campaign_experiment_service_pb2",
    'PromotionExtensionDiscountModifierEnum':"google.ads.google_ads.v4.proto.enums.promotion_extension_discount_modifier_pb2",
    'PromotionExtensionOccasionEnum':"google.ads.google_ads.v4.proto.enums.promotion_extension_occasion_pb2",
    'PromotionFeedItem':"google.ads.google_ads.v4.proto.common.extensions_pb2",
    'PromotionPlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.promotion_placeholder_field_pb2",
    'ProximityInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'ProximityRadiusUnitsEnum':"google.ads.google_ads.v4.proto.enums.proximity_radius_units_pb2",
    'QualityScoreBucketEnum':"google.ads.google_ads.v4.proto.enums.quality_score_bucket_pb2",
    'QueryErrorEnum':"google.ads.google_ads.v4.proto.errors.query_error_pb2",
    'QuotaErrorEnum':"google.ads.google_ads.v4.proto.errors.quota_error_pb2",
    'RangeErrorEnum':"google.ads.google_ads.v4.proto.errors.range_error_pb2",
    'ReachCurve':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'ReachForecast':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'ReachPlanAdLengthEnum':"google.ads.google_ads.v4.proto.enums.reach_plan_ad_length_pb2",
    'ReachPlanAgeRangeEnum':"google.ads.google_ads.v4.proto.enums.reach_plan_age_range_pb2",
    'ReachPlanErrorEnum':"google.ads.google_ads.v4.proto.errors.reach_plan_error_pb2",
    'ReachPlanNetworkEnum':"google.ads.google_ads.v4.proto.enums.reach_plan_network_pb2",
    'RealEstatePlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.real_estate_placeholder_field_pb2",
    'RealTimeBiddingSetting':"google.ads.google_ads.v4.proto.common.real_time_bidding_setting_pb2",
    'Recommendation':"google.ads.google_ads.v4.proto.resources.recommendation_pb2",
    'RecommendationErrorEnum':"google.ads.google_ads.v4.proto.errors.recommendation_error_pb2",
    'RecommendationTypeEnum':"google.ads.google_ads.v4.proto.enums.recommendation_type_pb2",
    'RegionCodeErrorEnum':"google.ads.google_ads.v4.proto.errors.region_code_error_pb2",
    'RemarketingAction':"google.ads.google_ads.v4.proto.resources.remarketing_action_pb2",
    'RemarketingActionOperation':"google.ads.google_ads.v4.proto.services.remarketing_action_service_pb2",
    'RemarketingSetting':"google.ads.google_ads.v4.proto.resources.customer_pb2",
    'RequestErrorEnum':"google.ads.google_ads.v4.proto.errors.request_error_pb2",
    'ResourceAccessDeniedErrorEnum':"google.ads.google_ads.v4.proto.errors.resource_access_denied_error_pb2",
    'ResourceCountLimitExceededErrorEnum':"google.ads.google_ads.v4.proto.errors.resource_count_limit_exceeded_error_pb2",
    'ResponsiveDisplayAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'ResponsiveSearchAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'RestatementValue':"google.ads.google_ads.v4.proto.services.conversion_adjustment_upload_service_pb2",
    'RuleBasedUserListInfo':"google.ads.google_ads.v4.proto.common.user_lists_pb2",
    'RunBatchJobRequest':"google.ads.google_ads.v4.proto.services.batch_job_service_pb2",
    'RunOfflineUserDataJobRequest':"google.ads.google_ads.v4.proto.services.offline_user_data_job_service_pb2",
    'SearchEngineResultsPageTypeEnum':"google.ads.google_ads.v4.proto.enums.search_engine_results_page_type_pb2",
    'SearchGoogleAdsFieldsRequest':"google.ads.google_ads.v4.proto.services.google_ads_field_service_pb2",
    'SearchGoogleAdsFieldsResponse':"google.ads.google_ads.v4.proto.services.google_ads_field_service_pb2",
    'SearchGoogleAdsRequest':"google.ads.google_ads.v4.proto.services.google_ads_service_pb2",
    'SearchGoogleAdsResponse':"google.ads.google_ads.v4.proto.services.google_ads_service_pb2",
    'SearchGoogleAdsStreamRequest':"google.ads.google_ads.v4.proto.services.google_ads_service_pb2",
    'SearchGoogleAdsStreamResponse':"google.ads.google_ads.v4.proto.services.google_ads_service_pb2",
    'SearchTermMatchTypeEnum':"google.ads.google_ads.v4.proto.enums.search_term_match_type_pb2",
    'SearchTermTargetingStatusEnum':"google.ads.google_ads.v4.proto.enums.search_term_targeting_status_pb2",
    'SearchTermView':"google.ads.google_ads.v4.proto.resources.search_term_view_pb2",
    'Segments':"google.ads.google_ads.v4.proto.common.segments_pb2",
    'ServedAssetFieldTypeEnum':"google.ads.google_ads.v4.proto.enums.served_asset_field_type_pb2",
    'SettingErrorEnum':"google.ads.google_ads.v4.proto.errors.setting_error_pb2",
    'SharedCriterion':"google.ads.google_ads.v4.proto.resources.shared_criterion_pb2",
    'SharedCriterionErrorEnum':"google.ads.google_ads.v4.proto.errors.shared_criterion_error_pb2",
    'SharedCriterionOperation':"google.ads.google_ads.v4.proto.services.shared_criterion_service_pb2",
    'SharedSet':"google.ads.google_ads.v4.proto.resources.shared_set_pb2",
    'SharedSetErrorEnum':"google.ads.google_ads.v4.proto.errors.shared_set_error_pb2",
    'SharedSetOperation':"google.ads.google_ads.v4.proto.services.shared_set_service_pb2",
    'SharedSetStatusEnum':"google.ads.google_ads.v4.proto.enums.shared_set_status_pb2",
    'SharedSetTypeEnum':"google.ads.google_ads.v4.proto.enums.shared_set_type_pb2",
    'ShoppingComparisonListingAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'ShoppingPerformanceView':"google.ads.google_ads.v4.proto.resources.shopping_performance_view_pb2",
    'ShoppingProductAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'ShoppingSmartAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'SimilarUserListInfo':"google.ads.google_ads.v4.proto.common.user_lists_pb2",
    'SimulationModificationMethodEnum':"google.ads.google_ads.v4.proto.enums.simulation_modification_method_pb2",
    'SimulationTypeEnum':"google.ads.google_ads.v4.proto.enums.simulation_type_pb2",
    'SiteSeed':"google.ads.google_ads.v4.proto.services.keyword_plan_idea_service_pb2",
    'SitelinkFeedItem':"google.ads.google_ads.v4.proto.common.extensions_pb2",
    'SitelinkPlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.sitelink_placeholder_field_pb2",
    'SizeLimitErrorEnum':"google.ads.google_ads.v4.proto.errors.size_limit_error_pb2",
    'SlotEnum':"google.ads.google_ads.v4.proto.enums.slot_pb2",
    'SpendingLimitTypeEnum':"google.ads.google_ads.v4.proto.enums.spending_limit_type_pb2",
    'StoreAttribute':"google.ads.google_ads.v4.proto.common.offline_user_data_pb2",
    'StoreSalesMetadata':"google.ads.google_ads.v4.proto.common.offline_user_data_pb2",
    'StoreSalesThirdPartyMetadata':"google.ads.google_ads.v4.proto.common.offline_user_data_pb2",
    'StringFormatErrorEnum':"google.ads.google_ads.v4.proto.errors.string_format_error_pb2",
    'StringLengthErrorEnum':"google.ads.google_ads.v4.proto.errors.string_length_error_pb2",
    'StructuredSnippetFeedItem':"google.ads.google_ads.v4.proto.common.extensions_pb2",
    'StructuredSnippetPlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.structured_snippet_placeholder_field_pb2",
    'SuggestGeoTargetConstantsRequest':"google.ads.google_ads.v4.proto.services.geo_target_constant_service_pb2",
    'SuggestGeoTargetConstantsResponse':"google.ads.google_ads.v4.proto.services.geo_target_constant_service_pb2",
    'SummaryRowSettingEnum':"google.ads.google_ads.v4.proto.enums.summary_row_setting_pb2",
    'SystemManagedResourceSourceEnum':"google.ads.google_ads.v4.proto.enums.system_managed_entity_source_pb2",
    'TagSnippet':"google.ads.google_ads.v4.proto.common.tag_snippet_pb2",
    'TargetCpa':"google.ads.google_ads.v4.proto.common.bidding_pb2",
    'TargetCpaOptInRecommendationGoalEnum':"google.ads.google_ads.v4.proto.enums.target_cpa_opt_in_recommendation_goal_pb2",
    'TargetCpaSimulationPoint':"google.ads.google_ads.v4.proto.common.simulation_pb2",
    'TargetCpaSimulationPointList':"google.ads.google_ads.v4.proto.common.simulation_pb2",
    'TargetCpm':"google.ads.google_ads.v4.proto.common.bidding_pb2",
    'TargetImpressionShare':"google.ads.google_ads.v4.proto.common.bidding_pb2",
    'TargetImpressionShareLocationEnum':"google.ads.google_ads.v4.proto.enums.target_impression_share_location_pb2",
    'TargetRestriction':"google.ads.google_ads.v4.proto.common.targeting_setting_pb2",
    'TargetRestrictionOperation':"google.ads.google_ads.v4.proto.common.targeting_setting_pb2",
    'TargetRoas':"google.ads.google_ads.v4.proto.common.bidding_pb2",
    'TargetRoasSimulationPoint':"google.ads.google_ads.v4.proto.common.simulation_pb2",
    'TargetRoasSimulationPointList':"google.ads.google_ads.v4.proto.common.simulation_pb2",
    'TargetSpend':"google.ads.google_ads.v4.proto.common.bidding_pb2",
    'Targeting':"google.ads.google_ads.v4.proto.services.reach_plan_service_pb2",
    'TargetingDimensionEnum':"google.ads.google_ads.v4.proto.enums.targeting_dimension_pb2",
    'TargetingSetting':"google.ads.google_ads.v4.proto.common.targeting_setting_pb2",
    'TextAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'TextAsset':"google.ads.google_ads.v4.proto.common.asset_types_pb2",
    'TextLabel':"google.ads.google_ads.v4.proto.common.text_label_pb2",
    'TextMessageFeedItem':"google.ads.google_ads.v4.proto.common.extensions_pb2",
    'ThirdPartyAppAnalyticsLink':"google.ads.google_ads.v4.proto.resources.third_party_app_analytics_link_pb2",
    'ThirdPartyAppAnalyticsLinkIdentifier':"google.ads.google_ads.v4.proto.resources.account_link_pb2",
    'TimeTypeEnum':"google.ads.google_ads.v4.proto.enums.time_type_pb2",
    'TimeZoneErrorEnum':"google.ads.google_ads.v4.proto.errors.time_zone_error_pb2",
    'TopicConstant':"google.ads.google_ads.v4.proto.resources.topic_constant_pb2",
    'TopicInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'TopicView':"google.ads.google_ads.v4.proto.resources.topic_view_pb2",
    'TrackingCodePageFormatEnum':"google.ads.google_ads.v4.proto.enums.tracking_code_page_format_pb2",
    'TrackingCodeTypeEnum':"google.ads.google_ads.v4.proto.enums.tracking_code_type_pb2",
    'TransactionAttribute':"google.ads.google_ads.v4.proto.common.offline_user_data_pb2",
    'TravelPlaceholderFieldEnum':"google.ads.google_ads.v4.proto.enums.travel_placeholder_field_pb2",
    'UnknownListingDimensionInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'UploadCallConversionsRequest':"google.ads.google_ads.v4.proto.services.conversion_upload_service_pb2",
    'UploadCallConversionsResponse':"google.ads.google_ads.v4.proto.services.conversion_upload_service_pb2",
    'UploadClickConversionsRequest':"google.ads.google_ads.v4.proto.services.conversion_upload_service_pb2",
    'UploadClickConversionsResponse':"google.ads.google_ads.v4.proto.services.conversion_upload_service_pb2",
    'UploadConversionAdjustmentsRequest':"google.ads.google_ads.v4.proto.services.conversion_adjustment_upload_service_pb2",
    'UploadConversionAdjustmentsResponse':"google.ads.google_ads.v4.proto.services.conversion_adjustment_upload_service_pb2",
    'UploadUserDataRequest':"google.ads.google_ads.v4.proto.services.user_data_service_pb2",
    'UploadUserDataResponse':"google.ads.google_ads.v4.proto.services.user_data_service_pb2",
    'UrlCollection':"google.ads.google_ads.v4.proto.common.url_collection_pb2",
    'UrlFieldErrorEnum':"google.ads.google_ads.v4.proto.errors.url_field_error_pb2",
    'UrlSeed':"google.ads.google_ads.v4.proto.services.keyword_plan_idea_service_pb2",
    'UserData':"google.ads.google_ads.v4.proto.common.offline_user_data_pb2",
    'UserDataErrorEnum':"google.ads.google_ads.v4.proto.errors.user_data_error_pb2",
    'UserDataOperation':"google.ads.google_ads.v4.proto.services.user_data_service_pb2",
    'UserIdentifier':"google.ads.google_ads.v4.proto.common.offline_user_data_pb2",
    'UserInterest':"google.ads.google_ads.v4.proto.resources.user_interest_pb2",
    'UserInterestInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'UserInterestTaxonomyTypeEnum':"google.ads.google_ads.v4.proto.enums.user_interest_taxonomy_type_pb2",
    'UserList':"google.ads.google_ads.v4.proto.resources.user_list_pb2",
    'UserListAccessStatusEnum':"google.ads.google_ads.v4.proto.enums.user_list_access_status_pb2",
    'UserListActionInfo':"google.ads.google_ads.v4.proto.common.user_lists_pb2",
    'UserListClosingReasonEnum':"google.ads.google_ads.v4.proto.enums.user_list_closing_reason_pb2",
    'UserListCombinedRuleOperatorEnum':"google.ads.google_ads.v4.proto.enums.user_list_combined_rule_operator_pb2",
    'UserListCrmDataSourceTypeEnum':"google.ads.google_ads.v4.proto.enums.user_list_crm_data_source_type_pb2",
    'UserListDateRuleItemInfo':"google.ads.google_ads.v4.proto.common.user_lists_pb2",
    'UserListDateRuleItemOperatorEnum':"google.ads.google_ads.v4.proto.enums.user_list_date_rule_item_operator_pb2",
    'UserListErrorEnum':"google.ads.google_ads.v4.proto.errors.user_list_error_pb2",
    'UserListInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'UserListLogicalRuleInfo':"google.ads.google_ads.v4.proto.common.user_lists_pb2",
    'UserListLogicalRuleOperatorEnum':"google.ads.google_ads.v4.proto.enums.user_list_logical_rule_operator_pb2",
    'UserListMembershipStatusEnum':"google.ads.google_ads.v4.proto.enums.user_list_membership_status_pb2",
    'UserListNumberRuleItemInfo':"google.ads.google_ads.v4.proto.common.user_lists_pb2",
    'UserListNumberRuleItemOperatorEnum':"google.ads.google_ads.v4.proto.enums.user_list_number_rule_item_operator_pb2",
    'UserListOperation':"google.ads.google_ads.v4.proto.services.user_list_service_pb2",
    'UserListPrepopulationStatusEnum':"google.ads.google_ads.v4.proto.enums.user_list_prepopulation_status_pb2",
    'UserListRuleInfo':"google.ads.google_ads.v4.proto.common.user_lists_pb2",
    'UserListRuleItemGroupInfo':"google.ads.google_ads.v4.proto.common.user_lists_pb2",
    'UserListRuleItemInfo':"google.ads.google_ads.v4.proto.common.user_lists_pb2",
    'UserListRuleTypeEnum':"google.ads.google_ads.v4.proto.enums.user_list_rule_type_pb2",
    'UserListSizeRangeEnum':"google.ads.google_ads.v4.proto.enums.user_list_size_range_pb2",
    'UserListStringRuleItemInfo':"google.ads.google_ads.v4.proto.common.user_lists_pb2",
    'UserListStringRuleItemOperatorEnum':"google.ads.google_ads.v4.proto.enums.user_list_string_rule_item_operator_pb2",
    'UserListTypeEnum':"google.ads.google_ads.v4.proto.enums.user_list_type_pb2",
    'UserLocationView':"google.ads.google_ads.v4.proto.resources.user_location_view_pb2",
    'Value':"google.ads.google_ads.v4.proto.common.value_pb2",
    'VanityPharmaDisplayUrlModeEnum':"google.ads.google_ads.v4.proto.enums.vanity_pharma_display_url_mode_pb2",
    'VanityPharmaTextEnum':"google.ads.google_ads.v4.proto.enums.vanity_pharma_text_pb2",
    'Video':"google.ads.google_ads.v4.proto.resources.video_pb2",
    'VideoAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'VideoBumperInStreamAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'VideoNonSkippableInStreamAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'VideoOutstreamAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'VideoTrueViewDiscoveryAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'VideoTrueViewInStreamAdInfo':"google.ads.google_ads.v4.proto.common.ad_type_infos_pb2",
    'WebpageConditionInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'WebpageConditionOperandEnum':"google.ads.google_ads.v4.proto.enums.webpage_condition_operand_pb2",
    'WebpageConditionOperatorEnum':"google.ads.google_ads.v4.proto.enums.webpage_condition_operator_pb2",
    'WebpageInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'YouTubeChannelInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'YouTubeVideoInfo':"google.ads.google_ads.v4.proto.common.criteria_pb2",
    'YoutubeVideoAsset':"google.ads.google_ads.v4.proto.common.asset_types_pb2",
    'YoutubeVideoRegistrationErrorEnum':"google.ads.google_ads.v4.proto.errors.youtube_video_registration_error_pb2",
}


DEPENDENT_MODULE_LIST = [
    "google.longrunning.operations_pb2",
    "google.protobuf.any_pb2",
    "google.protobuf.empty_pb2",
    "google.protobuf.field_mask_pb2",
    "google.protobuf.wrappers_pb2",
    "google.rpc.status_pb2",
]


def _get_class_from_module(module_name):
    module = importlib.import_module(module_name)
    for class_name in get_messages(module).keys():  # from inspect module
        yield class_name


def _populate_dependent_classes(module_list=DEPENDENT_MODULE_LIST):
    class_list = {}
    for module_name in module_list:
        for cls in _get_class_from_module(module_name):
            class_list[cls] = module_name
    return class_list


_lazy_dependent_class_to_package_map = _populate_dependent_classes()


def _load_module(module_name):
    """Load a module by it's name.

    Args:
        module_name: a str of the name of a sub-module to load.

    Returns:
        A module class instance.

    Raises:
        AttributeError if the given module can't be found.
    """
    try:
        if module_name in _lazy_name_to_package_map:
            module_path = (
                f"{_lazy_name_to_package_map[module_name]}.{module_name}"
            )
        else:
            module_path = module_name
        return importlib.import_module(module_path)
    except KeyError:
        raise AttributeError(f"unknown sub-module {module_name!r}.")


def _get_module_by_name(module_name):
    """Get a module containing one or more message classes.

    For example: google.ads.google_ads.v2.proto.services.video_service_pb2.

    Args:
        module_name: a str of the name of a module.

    Returns:
        a module class instance.
    """
    module = _load_module(module_name)
    globals()[module_name] = module

    for name, message in get_messages(module).items():
        if name.endswith("_service_pb2"):
            message.__module__ = "google.ads.google_ads.v2.types"
        globals()[name] = message

    return module


def _get_message_class_by_name(class_name):
    """Get a message class instance by name.

    For example: VideoService

    Args:
        module_name: a str of the name of a protobuf class to load.

    Returns:
        a protobuf message class definition that inherits from
            google.protobuf.pyext.cpp_message.GeneratedProtocolMessageType.
    """
    if class_name in _lazy_dependent_class_to_package_map:
        module_path = _lazy_dependent_class_to_package_map[class_name]
    elif class_name in _lazy_class_to_package_map:
        module_path = _lazy_class_to_package_map[class_name]
    else:
        raise AttributeError(f"unknown sub-module {class_name!r}.")

    try:
        module = _load_module(module_path)
        message = getattr(module, class_name)
    except AttributeError:
        raise AttributeError(f"unknown message class {class_name!r}.")

    if class_name.endswith("Service"):
        message.__module__ = "google.ads.google_ads.v2.types"

    globals()[class_name] = message
    return message


# Background on how this behaves: https://www.python.org/dev/peps/pep-0562/
def __getattr__(name):  # Requires Python >= 3.7
    """Lazily perform imports and class definitions on first demand."""
    if name == "__all__":
        converted = (
            util.convert_snake_case_to_upper_case(key)
            for key in chain(
                _lazy_name_to_package_map,
                _lazy_class_to_package_map,
                _lazy_dependent_class_to_package_map,
            )
        )
        all_names = sorted(converted)
        globals()["__all__"] = all_names
        return all_names
    elif name.endswith("_pb2"):
        return _get_module_by_name(name)
    elif name.endswith("Pb2"):
        module_name = f"{util.convert_upper_case_to_snake_case(name)}"
        return _get_module_by_name(module_name)
    else:
        return _get_message_class_by_name(name)


def __dir__():
    return globals().get("__all__") or __getattr__("__all__")


if not sys.version_info >= (3, 7):
    from pep562 import Pep562

    Pep562(__name__)
